# TestSwiftRemoteASTImport.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2018 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2

class TestSwiftRemoteASTImport(TestBase):
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipUnlessDarwin
    @swiftTest
    def testSwiftRemoteASTImport(self):
        """This tests that RemoteAST querying the dynamic type of a variable
        doesn't import any modules into a module SwiftASTContext that
        weren't imported by that module in the source code.

        """
        self.build()

        lldbutil.run_to_source_breakpoint(self, "break here",
                                          lldb.SBFileSpec('Library.swift'),
                                          extra_images=['Library'])
        # FIXME: Reversing the order of these two commands does not work!
        self.expect("expr -d no-dynamic-values -- input",
                    substrs=['(Library.LibraryProtocol) $R0'])
        self.expect("expr -d run-target -- input",
                    substrs=['(a.FromMainModule) $R1'])



"""
Test lldb settings command.
"""

import json
import os
import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SettingsCommandTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_apropos_should_also_search_settings_description(self):
        """Test that 'apropos' command should also search descriptions for the settings variables."""

        self.expect(
            "apropos 'environment variable'",
            substrs=[
                "target.env-vars",
                "environment variables",
                "executable's environment",
            ],
        )

    def test_set_interpreter_repeat_prev_command(self):
        """Test the `interpreter.repeat-previous-command` setting."""
        self.build()

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)
        setting = "interpreter.repeat-previous-command"

        def cleanup(setting):
            self.runCmd("settings clear %s" % setting, check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup(setting))

        # First, check for the setting default value.
        self.expect(
            "setting show %s" % setting,
            substrs=["interpreter.repeat-previous-command (boolean) = true"],
        )

        # Then, invert the setting, and check that was set correctly
        self.runCmd("setting set %s false" % setting)
        self.expect(
            "setting show %s" % setting,
            substrs=["interpreter.repeat-previous-command (boolean) = false"],
        )

        ci = self.dbg.GetCommandInterpreter()
        self.assertTrue(ci.IsValid(), "Invalid command interpreter.")
        # Now, test the functionnality
        res = lldb.SBCommandReturnObject()
        ci.HandleCommand("breakpoint set -n main", res)
        self.assertTrue(res.Succeeded(), "Command failed.")
        ci.HandleCommand("", res)
        self.assertTrue(res.Succeeded(), "Empty command failed.")
        self.assertEqual(self.dbg.GetSelectedTarget().GetNumBreakpoints(), 1)

    def test_append_target_env_vars(self):
        """Test that 'append target.env-vars' works."""
        # Append the env-vars.
        self.runCmd("settings append target.env-vars MY_ENV_VAR=YES")
        # And add hooks to restore the settings during tearDown().
        self.addTearDownHook(lambda: self.runCmd("settings clear target.env-vars"))

        # Check it immediately!
        self.expect("settings show target.env-vars", substrs=["MY_ENV_VAR=YES"])

    def test_insert_before_and_after_target_run_args(self):
        """Test that 'insert-before/after target.run-args' works."""
        # Set the run-args first.
        self.runCmd("settings set target.run-args a b c")
        # And add hooks to restore the settings during tearDown().
        self.addTearDownHook(lambda: self.runCmd("settings clear target.run-args"))

        # Now insert-before the index-0 element with '__a__'.
        self.runCmd("settings insert-before target.run-args 0 __a__")
        # And insert-after the index-1 element with '__A__'.
        self.runCmd("settings insert-after target.run-args 1 __A__")
        # Check it immediately!
        self.expect(
            "settings show target.run-args",
            substrs=[
                "target.run-args",
                '[0]: "__a__"',
                '[1]: "a"',
                '[2]: "__A__"',
                '[3]: "b"',
                '[4]: "c"',
            ],
        )

    def test_replace_target_run_args(self):
        """Test that 'replace target.run-args' works."""
        # Set the run-args and then replace the index-0 element.
        self.runCmd("settings set target.run-args a b c")
        # And add hooks to restore the settings during tearDown().
        self.addTearDownHook(lambda: self.runCmd("settings clear target.run-args"))

        # Now replace the index-0 element with 'A', instead.
        self.runCmd("settings replace target.run-args 0 A")
        # Check it immediately!
        self.expect(
            "settings show target.run-args",
            substrs=[
                "target.run-args (arguments) =",
                '[0]: "A"',
                '[1]: "b"',
                '[2]: "c"',
            ],
        )

    def test_set_prompt(self):
        """Test that 'set prompt' actually changes the prompt."""

        # Set prompt to 'lldb2'.
        self.runCmd("settings set prompt 'lldb2 '")

        # Immediately test the setting.
        self.expect(
            "settings show prompt",
            SETTING_MSG("prompt"),
            startstr='prompt (string) = "lldb2 "',
        )

        # The overall display should also reflect the new setting.
        self.expect(
            "settings show",
            SETTING_MSG("prompt"),
            substrs=['prompt (string) = "lldb2 "'],
        )

        # Use '-r' option to reset to the original default prompt.
        self.runCmd("settings clear prompt")

    def test_set_term_width(self):
        """Test that 'set term-width' actually changes the term-width."""

        self.runCmd("settings set term-width 70")

        # Immediately test the setting.
        self.expect(
            "settings show term-width",
            SETTING_MSG("term-width"),
            startstr="term-width (unsigned) = 70",
        )

        # The overall display should also reflect the new setting.
        self.expect(
            "settings show",
            SETTING_MSG("term-width"),
            substrs=["term-width (unsigned) = 70"],
        )

        self.dbg.SetTerminalWidth(60)

        self.expect(
            "settings show",
            SETTING_MSG("term-width"),
            substrs=["term-width (unsigned) = 60"],
        )

    # rdar://problem/10712130
    @skipIf(oslist=["windows"], bugnumber="llvm.org/pr44431")
    def test_set_frame_format(self):
        """Test that 'set frame-format' with a backtick char in the format string works as well as fullpath."""
        self.build()

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        def cleanup():
            self.runCmd(
                "settings set frame-format %s" % self.format_string, check=False
            )

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        self.runCmd("settings show frame-format")
        m = re.match('^frame-format \(format-string\) = "(.*)"$', self.res.GetOutput())
        self.assertTrue(m, "Bad settings string")
        self.format_string = m.group(1)

        # Change the default format to print function.name rather than
        # function.name-with-args
        format_string = "frame #${frame.index}: ${frame.pc}{ ${module.file.basename}\`${function.name}{${function.pc-offset}}}{ at ${line.file.fullpath}:${line.number}}{, lang=${language}}\n"
        self.runCmd("settings set frame-format %s" % format_string)

        # Immediately test the setting.
        self.expect(
            "settings show frame-format",
            SETTING_MSG("frame-format"),
            substrs=[format_string],
        )

        self.runCmd("breakpoint set -n main")
        self.runCmd(
            "process launch --working-dir '{0}'".format(
                self.get_process_working_directory()
            ),
            RUN_SUCCEEDED,
        )
        self.expect("thread backtrace", substrs=["`main", self.getSourceDir()])

    def test_set_auto_confirm(self):
        """Test that after 'set auto-confirm true', manual confirmation should not kick in."""
        self.build()

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        self.runCmd("settings set auto-confirm true")

        # Immediately test the setting.
        self.expect(
            "settings show auto-confirm",
            SETTING_MSG("auto-confirm"),
            startstr="auto-confirm (boolean) = true",
        )

        # Now 'breakpoint delete' should just work fine without confirmation
        # prompt from the command interpreter.
        self.runCmd("breakpoint set -n main")
        self.expect("breakpoint delete", startstr="All breakpoints removed")

        # Restore the original setting of auto-confirm.
        self.runCmd("settings clear auto-confirm")
        self.expect(
            "settings show auto-confirm",
            SETTING_MSG("auto-confirm"),
            startstr="auto-confirm (boolean) = false",
        )

    @skipIf(archs=no_match(["x86_64", "i386", "i686"]))
    def test_disassembler_settings(self):
        """Test that user options for the disassembler take effect."""
        self.build()

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # AT&T syntax
        self.runCmd("settings set target.x86-disassembly-flavor att")
        self.runCmd("settings set target.use-hex-immediates false")
        self.expect("disassemble -n numberfn", substrs=["$90"])
        self.runCmd("settings set target.use-hex-immediates true")
        self.runCmd("settings set target.hex-immediate-style c")
        self.expect("disassemble -n numberfn", substrs=["$0x5a"])
        self.runCmd("settings set target.hex-immediate-style asm")
        self.expect("disassemble -n numberfn", substrs=["$5ah"])

        # Intel syntax
        self.runCmd("settings set target.x86-disassembly-flavor intel")
        self.runCmd("settings set target.use-hex-immediates false")
        self.expect("disassemble -n numberfn", substrs=["90"])
        self.runCmd("settings set target.use-hex-immediates true")
        self.runCmd("settings set target.hex-immediate-style c")
        self.expect("disassemble -n numberfn", substrs=["0x5a"])
        self.runCmd("settings set target.hex-immediate-style asm")
        self.expect("disassemble -n numberfn", substrs=["5ah"])

    @skipIfDarwinEmbedded  # <rdar://problem/34446098> debugserver on ios etc can't write files
    def test_run_args_and_env_vars(self):
        self.do_test_run_args_and_env_vars(use_launchsimple=False)

    @skipIfDarwinEmbedded  # <rdar://problem/34446098> debugserver on ios etc can't write files
    def test_launchsimple_args_and_env_vars(self):
        self.do_test_run_args_and_env_vars(use_launchsimple=True)

    def do_test_run_args_and_env_vars(self, use_launchsimple):
        """Test that run-args and env-vars are passed to the launched process."""
        self.build()

        # Set the run-args and the env-vars.
        # And add hooks to restore the settings during tearDown().
        self.runCmd("settings set target.run-args A B C")
        self.addTearDownHook(lambda: self.runCmd("settings clear target.run-args"))
        self.runCmd('settings set target.env-vars ["MY_ENV_VAR"]=YES')
        self.addTearDownHook(lambda: self.runCmd("settings clear target.env-vars"))

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        target = self.dbg.GetTargetAtIndex(0)
        launch_info = target.GetLaunchInfo()
        found_env_var = False
        for i in range(0, launch_info.GetNumEnvironmentEntries()):
            if launch_info.GetEnvironmentEntryAtIndex(i) == "MY_ENV_VAR=YES":
                found_env_var = True
                break
        self.assertTrue(found_env_var, "MY_ENV_VAR was not set in LaunchInfo object")

        self.assertEqual(launch_info.GetNumArguments(), 3)
        self.assertEqual(launch_info.GetArgumentAtIndex(0), "A")
        self.assertEqual(launch_info.GetArgumentAtIndex(1), "B")
        self.assertEqual(launch_info.GetArgumentAtIndex(2), "C")

        self.expect("target show-launch-environment", substrs=["MY_ENV_VAR=YES"])

        wd = self.get_process_working_directory()
        if use_launchsimple:
            process = target.LaunchSimple(None, None, wd)
            self.assertTrue(process)
        else:
            self.runCmd("process launch --working-dir '{0}'".format(wd), RUN_SUCCEEDED)

        # Read the output file produced by running the program.
        output = lldbutil.read_file_from_process_wd(self, "output2.txt")

        self.expect(
            output,
            exe=False,
            substrs=[
                "argv[1] matches",
                "argv[2] matches",
                "argv[3] matches",
                "Environment variable 'MY_ENV_VAR' successfully passed.",
            ],
        )

        # Check that env-vars overrides unset-env-vars.
        self.runCmd("settings set target.unset-env-vars MY_ENV_VAR")

        self.expect(
            "target show-launch-environment",
            "env-vars overrides unset-env-vars",
            substrs=["MY_ENV_VAR=YES"],
        )

        wd = self.get_process_working_directory()
        if use_launchsimple:
            process = target.LaunchSimple(None, None, wd)
            self.assertTrue(process)
        else:
            self.runCmd("process launch --working-dir '{0}'".format(wd), RUN_SUCCEEDED)

        # Read the output file produced by running the program.
        output = lldbutil.read_file_from_process_wd(self, "output2.txt")

        self.expect(
            output,
            exe=False,
            substrs=["Environment variable 'MY_ENV_VAR' successfully passed."],
        )

    @skipIfRemote  # it doesn't make sense to send host env to remote target
    def test_pass_host_env_vars(self):
        """Test that the host env vars are passed to the launched process."""
        self.build()

        # Set some host environment variables now.
        os.environ["MY_HOST_ENV_VAR1"] = "VAR1"
        os.environ["MY_HOST_ENV_VAR2"] = "VAR2"

        # This is the function to unset the two env variables set above.
        def unset_env_variables():
            os.environ.pop("MY_HOST_ENV_VAR1")
            os.environ.pop("MY_HOST_ENV_VAR2")

        self.addTearDownHook(unset_env_variables)

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # By default, inherit-env is 'true'.
        self.expect(
            "settings show target.inherit-env",
            "Default inherit-env is 'true'",
            startstr="target.inherit-env (boolean) = true",
        )

        self.expect(
            "target show-launch-environment",
            "Host environment is passed correctly",
            substrs=["MY_HOST_ENV_VAR1=VAR1", "MY_HOST_ENV_VAR2=VAR2"],
        )
        self.runCmd(
            "process launch --working-dir '{0}'".format(
                self.get_process_working_directory()
            ),
            RUN_SUCCEEDED,
        )

        # Read the output file produced by running the program.
        output = lldbutil.read_file_from_process_wd(self, "output1.txt")

        self.expect(
            output,
            exe=False,
            substrs=[
                "The host environment variable 'MY_HOST_ENV_VAR1' successfully passed.",
                "The host environment variable 'MY_HOST_ENV_VAR2' successfully passed.",
            ],
        )

        # Now test that we can prevent the inferior from inheriting the
        # environment.
        self.runCmd("settings set target.inherit-env false")

        self.expect(
            "target show-launch-environment",
            "target.inherit-env affects `target show-launch-environment`",
            matching=False,
            substrs=["MY_HOST_ENV_VAR1=VAR1", "MY_HOST_ENV_VAR2=VAR2"],
        )

        self.runCmd(
            "process launch --working-dir '{0}'".format(
                self.get_process_working_directory()
            ),
            RUN_SUCCEEDED,
        )

        # Read the output file produced by running the program.
        output = lldbutil.read_file_from_process_wd(self, "output1.txt")

        self.expect(
            output,
            exe=False,
            matching=False,
            substrs=[
                "The host environment variable 'MY_HOST_ENV_VAR1' successfully passed.",
                "The host environment variable 'MY_HOST_ENV_VAR2' successfully passed.",
            ],
        )

        # Now test that we can unset variables from the inherited environment.
        self.runCmd("settings set target.inherit-env true")
        self.runCmd("settings set target.unset-env-vars MY_HOST_ENV_VAR1")
        self.runCmd(
            "process launch --working-dir '{0}'".format(
                self.get_process_working_directory()
            ),
            RUN_SUCCEEDED,
        )

        # Read the output file produced by running the program.
        output = lldbutil.read_file_from_process_wd(self, "output1.txt")

        self.expect(
            "target show-launch-environment",
            "MY_HOST_ENV_VAR1 is unset, it shouldn't be in `target show-launch-environment`",
            matching=False,
            substrs=["MY_HOST_ENV_VAR1=VAR1"],
        )
        self.expect(
            "target show-launch-environment",
            "MY_HOST_ENV_VAR2 shouldn be in `target show-launch-environment`",
            substrs=["MY_HOST_ENV_VAR2=VAR2"],
        )

        self.expect(
            output,
            exe=False,
            matching=False,
            substrs=[
                "The host environment variable 'MY_HOST_ENV_VAR1' successfully passed."
            ],
        )
        self.expect(
            output,
            exe=False,
            substrs=[
                "The host environment variable 'MY_HOST_ENV_VAR2' successfully passed."
            ],
        )

    @skipIfDarwinEmbedded  # <rdar://problem/34446098> debugserver on ios etc can't write files
    def test_set_error_output_path(self):
        """Test that setting target.error/output-path for the launched process works."""
        self.build()

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Set the error-path and output-path and verify both are set.
        self.runCmd(
            "settings set target.error-path '{0}'".format(
                lldbutil.append_to_process_working_directory(self, "stderr.txt")
            )
        )
        self.runCmd(
            "settings set target.output-path '{0}".format(
                lldbutil.append_to_process_working_directory(self, "stdout.txt")
            )
        )
        # And add hooks to restore the original settings during tearDown().
        self.addTearDownHook(lambda: self.runCmd("settings clear target.output-path"))
        self.addTearDownHook(lambda: self.runCmd("settings clear target.error-path"))

        self.expect(
            "settings show target.error-path",
            SETTING_MSG("target.error-path"),
            substrs=["target.error-path (file)", 'stderr.txt"'],
        )

        self.expect(
            "settings show target.output-path",
            SETTING_MSG("target.output-path"),
            substrs=["target.output-path (file)", 'stdout.txt"'],
        )

        self.runCmd(
            "process launch --working-dir '{0}'".format(
                self.get_process_working_directory()
            ),
            RUN_SUCCEEDED,
        )

        output = lldbutil.read_file_from_process_wd(self, "stderr.txt")
        message = "This message should go to standard error."
        if lldbplatformutil.hasChattyStderr(self):
            self.expect(output, exe=False, substrs=[message])
        else:
            self.expect(output, exe=False, startstr=message)

        output = lldbutil.read_file_from_process_wd(self, "stdout.txt")
        self.expect(
            output, exe=False, startstr="This message should go to standard out."
        )

    def test_print_dictionary_setting(self):
        self.runCmd("settings clear target.env-vars")
        self.runCmd('settings set target.env-vars ["MY_VAR"]=some-value')
        self.expect("settings show target.env-vars", substrs=["MY_VAR=some-value"])
        self.runCmd("settings clear target.env-vars")

    def test_print_array_setting(self):
        self.runCmd("settings clear target.run-args")
        self.runCmd("settings set target.run-args gobbledy-gook")
        self.expect("settings show target.run-args", substrs=['[0]: "gobbledy-gook"'])
        self.runCmd("settings clear target.run-args")

    def test_settings_with_quotes(self):
        self.runCmd("settings clear target.run-args")
        self.runCmd("settings set target.run-args a b c")
        self.expect(
            "settings show target.run-args",
            substrs=['[0]: "a"', '[1]: "b"', '[2]: "c"'],
        )
        self.runCmd("settings set target.run-args 'a b c'")
        self.expect("settings show target.run-args", substrs=['[0]: "a b c"'])
        self.runCmd("settings clear target.run-args")
        self.runCmd("settings clear target.env-vars")
        self.runCmd(
            'settings set target.env-vars ["MY_FILE"]="this is a file name with spaces.txt"'
        )
        self.expect(
            "settings show target.env-vars",
            substrs=["MY_FILE=this is a file name with spaces.txt"],
        )
        self.runCmd("settings clear target.env-vars")
        # Test and make sure that setting "format-string" settings obeys quotes
        # if they are provided
        self.runCmd("settings set thread-format    'abc def'   ")
        self.expect(
            "settings show thread-format",
            startstr='thread-format (format-string) = "abc def"',
        )
        self.runCmd('settings set thread-format    "abc def"   ')
        self.expect(
            "settings show thread-format",
            startstr='thread-format (format-string) = "abc def"',
        )
        # Make sure when no quotes are provided that we maintain any trailing
        # spaces
        self.runCmd("settings set thread-format abc def   ")
        self.expect(
            "settings show thread-format",
            startstr='thread-format (format-string) = "abc def   "',
        )
        self.runCmd("settings clear thread-format")

    def test_settings_with_trailing_whitespace(self):
        # boolean
        # Set to known value
        self.runCmd("settings set target.skip-prologue true")
        # Set to new value with trailing whitespace
        self.runCmd("settings set target.skip-prologue false ")
        # Make sure the setting was correctly set to "false"
        self.expect(
            "settings show target.skip-prologue",
            SETTING_MSG("target.skip-prologue"),
            startstr="target.skip-prologue (boolean) = false",
        )
        self.runCmd("settings clear target.skip-prologue", check=False)
        # integer
        self.runCmd("settings set term-width 70")  # Set to known value
        # Set to new value with trailing whitespaces
        self.runCmd("settings set term-width 60 \t")
        self.expect(
            "settings show term-width",
            SETTING_MSG("term-width"),
            startstr="term-width (unsigned) = 60",
        )
        self.runCmd("settings clear term-width", check=False)
        # string
        self.runCmd("settings set target.arg0 abc")  # Set to known value
        # Set to new value with trailing whitespaces
        self.runCmd("settings set target.arg0 cde\t ")
        self.expect(
            "settings show target.arg0",
            SETTING_MSG("target.arg0"),
            startstr='target.arg0 (string) = "cde"',
        )
        self.runCmd("settings clear target.arg0", check=False)
        # file
        path1 = self.getBuildArtifact("path1.txt")
        path2 = self.getBuildArtifact("path2.txt")
        self.runCmd("settings set target.output-path %s" % path1)  # Set to known value
        self.expect(
            "settings show target.output-path",
            SETTING_MSG("target.output-path"),
            startstr="target.output-path (file) = ",
            substrs=[path1],
        )
        self.runCmd(
            "settings set target.output-path %s " % path2
        )  # Set to new value with trailing whitespaces
        self.expect(
            "settings show target.output-path",
            SETTING_MSG("target.output-path"),
            startstr="target.output-path (file) = ",
            substrs=[path2],
        )
        self.runCmd("settings clear target.output-path", check=False)
        # enum
        # Set to known value
        self.runCmd("settings set stop-disassembly-display never")
        # Set to new value with trailing whitespaces
        self.runCmd("settings set stop-disassembly-display always ")
        self.expect(
            "settings show stop-disassembly-display",
            SETTING_MSG("stop-disassembly-display"),
            startstr="stop-disassembly-display (enum) = always",
        )
        self.runCmd("settings clear stop-disassembly-display", check=False)
        # language
        # Set to known value
        self.runCmd("settings set target.language c89")
        # Set to new value with trailing whitespace
        self.runCmd("settings set target.language c11 ")
        self.expect(
            "settings show target.language",
            SETTING_MSG("target.language"),
            startstr="target.language (language) = c11",
        )
        self.runCmd("settings clear target.language", check=False)
        # arguments
        self.runCmd("settings set target.run-args 1 2 3")  # Set to known value
        # Set to new value with trailing whitespaces
        self.runCmd("settings set target.run-args 3 4 5 ")
        self.expect(
            "settings show target.run-args",
            SETTING_MSG("target.run-args"),
            substrs=[
                "target.run-args (arguments) =",
                '[0]: "3"',
                '[1]: "4"',
                '[2]: "5"',
            ],
        )
        self.runCmd("settings set target.run-args 1 2 3")  # Set to known value
        # Set to new value with trailing whitespaces
        self.runCmd("settings set target.run-args 3 \  \ ")
        self.expect(
            "settings show target.run-args",
            SETTING_MSG("target.run-args"),
            substrs=[
                "target.run-args (arguments) =",
                '[0]: "3"',
                '[1]: " "',
                '[2]: " "',
            ],
        )
        self.runCmd("settings clear target.run-args", check=False)
        # dictionaries
        self.runCmd("settings clear target.env-vars")  # Set to known value
        # Set to new value with trailing whitespaces
        self.runCmd("settings set target.env-vars A=B C=D\t ")
        self.expect(
            "settings show target.env-vars",
            SETTING_MSG("target.env-vars"),
            substrs=["target.env-vars (dictionary of strings) =", "A=B", "C=D"],
        )
        self.runCmd("settings clear target.env-vars", check=False)
        # regex
        # Set to known value
        self.runCmd("settings clear target.process.thread.step-avoid-regexp")
        # Set to new value with trailing whitespaces
        self.runCmd("settings set target.process.thread.step-avoid-regexp foo\\ ")
        self.expect(
            "settings show target.process.thread.step-avoid-regexp",
            SETTING_MSG("target.process.thread.step-avoid-regexp"),
            substrs=["target.process.thread.step-avoid-regexp (regex) = foo\\ "],
        )
        self.runCmd(
            "settings clear target.process.thread.step-avoid-regexp", check=False
        )
        # format-string
        self.runCmd("settings clear disassembly-format")  # Set to known value
        # Set to new value with trailing whitespaces
        self.runCmd("settings set disassembly-format foo ")
        self.expect(
            "settings show disassembly-format",
            SETTING_MSG("disassembly-format"),
            substrs=['disassembly-format (format-string) = "foo "'],
        )
        self.runCmd("settings clear disassembly-format", check=False)

    def test_settings_list(self):
        # List settings (and optionally test the filter to only show 'target' settings).
        self.expect(
            "settings list target", substrs=["arg0", "detach-on-error", "language"]
        )
        self.expect("settings list target", matching=False, substrs=["packet-timeout"])
        self.expect(
            "settings list",
            substrs=["language", "arg0", "detach-on-error", "packet-timeout"],
        )

    def test_settings_remove_single(self):
        # Set some environment variables and use 'remove' to delete them.
        self.runCmd("settings set target.env-vars a=b c=d")
        self.expect("settings show target.env-vars", substrs=["a=b", "c=d"])
        self.runCmd("settings remove target.env-vars a")
        self.expect("settings show target.env-vars", matching=False, substrs=["a=b"])
        self.expect("settings show target.env-vars", substrs=["c=d"])
        self.runCmd("settings remove target.env-vars c")
        self.expect(
            "settings show target.env-vars", matching=False, substrs=["a=b", "c=d"]
        )

    def test_settings_remove_multiple(self):
        self.runCmd("settings set target.env-vars a=b c=d e=f")
        self.expect("settings show target.env-vars", substrs=["a=b", "c=d", "e=f"])
        self.runCmd("settings remove target.env-vars a e")
        self.expect(
            "settings show target.env-vars", matching=False, substrs=["a=b", "e=f"]
        )
        self.expect("settings show target.env-vars", substrs=["c=d"])

    def test_settings_remove_nonexistent_value(self):
        self.expect(
            "settings remove target.env-vars doesntexist",
            error=True,
            substrs=["no value found named 'doesntexist'"],
        )

    def test_settings_remove_nonexistent_settings(self):
        self.expect(
            "settings remove doesntexist alsodoesntexist",
            error=True,
            substrs=["error: invalid value path 'doesntexist'"],
        )

    def test_settings_remove_missing_arg(self):
        self.expect(
            "settings remove",
            error=True,
            substrs=["'settings remove' takes an array or dictionary item, or"],
        )

    def test_settings_remove_empty_arg(self):
        self.expect(
            "settings remove ''",
            error=True,
            substrs=["'settings remove' command requires a valid variable name"],
        )

    def test_settings_clear_all(self):
        # Change a dictionary.
        self.runCmd("settings set target.env-vars a=1 b=2 c=3")
        # Change an array.
        self.runCmd("settings set target.run-args a1 b2 c3")
        # Change a single boolean value.
        self.runCmd("settings set auto-confirm true")
        # Change a single integer value.
        self.runCmd("settings set tab-size 2")

        # Clear everything.
        self.runCmd("settings clear --all")

        # Check that settings have their default values after clearing.
        self.expect(
            "settings show target.env-vars",
            patterns=["^target.env-vars \(dictionary of strings\) =\s*$"],
        )
        self.expect(
            "settings show target.run-args",
            patterns=["^target.run-args \(arguments\) =\s*$"],
        )
        self.expect("settings show auto-confirm", substrs=["false"])
        self.expect("settings show tab-size", substrs=["4"])

        # Check that the command fails if we combine '--all' option with any arguments.
        self.expect(
            "settings clear --all auto-confirm",
            COMMAND_FAILED_AS_EXPECTED,
            error=True,
            substrs=["'settings clear --all' doesn't take any arguments"],
        )

    def test_all_settings_exist(self):
        self.expect(
            "settings show",
            substrs=[
                "auto-confirm",
                "frame-format",
                "notify-void",
                "prompt",
                "script-lang",
                "stop-disassembly-count",
                "stop-disassembly-display",
                "stop-line-count-after",
                "stop-line-count-before",
                "stop-show-column",
                "term-width",
                "thread-format",
                "use-external-editor",
                "target.breakpoints-use-platform-avoid-list",
                "target.default-arch",
                "target.disable-aslr",
                "target.disable-stdio",
                "target.x86-disassembly-flavor",
                "target.enable-synthetic-value",
                "target.env-vars",
                "target.error-path",
                "target.exec-search-paths",
                "target.expr-prefix",
                "target.hex-immediate-style",
                "target.inherit-env",
                "target.input-path",
                "target.language",
                "target.max-children-count",
                "target.max-string-summary-length",
                "target.move-to-nearest-code",
                "target.output-path",
                "target.prefer-dynamic-value",
                "target.run-args",
                "target.skip-prologue",
                "target.source-map",
                "target.use-hex-immediates",
                "target.process.disable-memory-cache",
                "target.process.extra-startup-command",
                "target.process.thread.trace-thread",
                "target.process.thread.step-avoid-regexp",
            ],
        )

    # settings under an ".experimental" domain should have two properties:
    #   1. If the name does not exist with "experimental" in the name path,
    #      the name lookup should try to find it without "experimental".  So
    #      a previously-experimental setting that has been promoted to a
    #      "real" setting will still be set by the original name.
    #   2. Changing a setting with .experimental., name, where the setting
    #      does not exist either with ".experimental." or without, should
    #      not generate an error.  So if an experimental setting is removed,
    #      people who may have that in their ~/.lldbinit files should not see
    #      any errors.
    def test_experimental_settings(self):
        cmdinterp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()

        # Set target.arg0 to a known value, check that we can retrieve it via
        # the actual name and via .experimental.
        self.expect("settings set target.arg0 first-value")
        self.expect("settings show target.arg0", substrs=["first-value"])
        self.expect(
            "settings show target.experimental.arg0",
            substrs=["first-value"],
            error=False,
        )

        # Set target.arg0 to a new value via a target.experimental.arg0 name,
        # verify that we can read it back via both .experimental., and not.
        self.expect("settings set target.experimental.arg0 second-value", error=False)
        self.expect("settings show target.arg0", substrs=["second-value"])
        self.expect(
            "settings show target.experimental.arg0",
            substrs=["second-value"],
            error=False,
        )

        # showing & setting an undefined .experimental. setting should generate no errors.
        self.expect(
            "settings show target.experimental.setting-which-does-not-exist",
            patterns=["^\s$"],
            error=False,
        )
        self.expect(
            "settings set target.experimental.setting-which-does-not-exist true",
            error=False,
        )

        # A domain component before .experimental. which does not exist should give an error
        # But the code does not yet do that.
        # self.expect('settings set target.setting-which-does-not-exist.experimental.arg0 true', error=True)

        # finally, confirm that trying to set a setting that does not exist still fails.
        # (SHOWING a setting that does not exist does not currently yield an error.)
        self.expect("settings set target.setting-which-does-not-exist true", error=True)

    def test_settings_set_exists(self):
        cmdinterp = self.dbg.GetCommandInterpreter()

        # An unknown option should succeed.
        self.expect("settings set -e foo bar")
        self.expect("settings set --exists foo bar")

        # A known option should fail if its argument is invalid.
        self.expect("settings set auto-confirm bogus", error=True)

    def get_setting_json(self, setting_path=None):
        settings_data = self.dbg.GetSetting(setting_path)
        stream = lldb.SBStream()
        settings_data.GetAsJSON(stream)
        return json.loads(stream.GetData())

    def verify_setting_value_json(self, setting_path, setting_value):
        self.runCmd("settings set %s %s" % (setting_path, setting_value))
        settings_json = self.get_setting_json(setting_path)
        self.assertEqual(settings_json, setting_value)

    def test_settings_api(self):
        """
        Test that ensures SBDebugger::GetSetting() APIs
        can correctly fetch settings.
        """

        # Test basic values and embedding special JSON escaping characters.
        self.runCmd("settings set auto-confirm true")
        self.runCmd("settings set tab-size 2")
        arg_value = 'hello "world"'
        self.runCmd("settings set target.arg0 %s" % arg_value)

        settings_json = self.get_setting_json()
        self.assertEqual(settings_json["auto-confirm"], True)
        self.assertEqual(settings_json["tab-size"], 2)
        self.assertEqual(settings_json["target"]["arg0"], arg_value)

        settings_data = self.get_setting_json("target.arg0")
        self.assertEqual(settings_data, arg_value)

        # Test OptionValueFileSpec
        self.verify_setting_value_json(
            "platform.module-cache-directory", self.get_process_working_directory()
        )

        # Test OptionValueArray
        setting_path = "target.run-args"
        setting_value = ["value1", "value2", "value3"]
        self.runCmd("settings set %s %s" % (setting_path, " ".join(setting_value)))
        settings_json = self.get_setting_json(setting_path)
        self.assertEqual(settings_json, setting_value)

        # Test OptionValueFormatEntity
        setting_value = """thread #${thread.index}{, name = \\'${thread.name}\\
        '}{, queue = ${ansi.fg.green}\\'${thread.queue}\\'${ansi.normal}}{,
        activity = ${ansi.fg.green}\\'${thread.info.activity.name}\\'${ansi.normal}}
        {, ${thread.info.trace_messages} messages}{, stop reason = ${ansi.fg.red}$
        {thread.stop-reason}${ansi.normal}}{\\\\nReturn value: ${thread.return-value}}
        {\\\\nCompleted expression: ${thread.completed-expression}}\\\\n"""
        self.verify_setting_value_json("thread-stop-format", setting_value)

        # Test OptionValueRegex
        self.verify_setting_value_json(
            "target.process.thread.step-avoid-regexp", "^std::"
        )

        # Test OptionValueLanguage
        self.verify_setting_value_json("repl-lang", "c++")
        
    def test_global_option(self):
        # This command used to crash the settings because -g was signaled by a
        # NULL execution context (not one with an empty Target...) and in the
        # special handling for load-script-from-symbol-file this wasn't checked.
        self.runCmd("settings set -g target.load-script-from-symbol-file true")

          



"""
Test basics of mach core file debugging.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class MachCoreTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # This was originally marked as expected failure on Windows, but it has
    # started timing out instead, so the expectedFailure attribute no longer
    # correctly tracks it: llvm.org/pr37371
    @skipIfWindows
    def test_selected_thread(self):
        """Test that the right thread is selected after a core is loaded."""
        # Create core form YAML.
        self.yaml2obj("test.core.yaml", self.getBuildArtifact("test.core"))

        # Set debugger into synchronous mode
        self.dbg.SetAsync(False)

        # Create a target by the debugger.
        target = self.dbg.CreateTarget("")

        # Load OS plugin.
        python_os_plugin_path = os.path.join(self.getSourceDir(), "operating_system.py")
        command = "settings set target.process.python-os-plugin-path '{}'".format(
            python_os_plugin_path
        )
        self.dbg.HandleCommand(command)

        # Load core.
        process = target.LoadCore(self.getBuildArtifact("test.core"))
        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(process.GetNumThreads(), 3)

        # Verify our OS plug-in threads showed up
        thread = process.GetThreadByID(0x111111111)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x111111111 after we load the python OS plug-in",
        )
        thread = process.GetThreadByID(0x222222222)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x222222222 after we load the python OS plug-in",
        )
        thread = process.GetThreadByID(0x333333333)
        self.assertTrue(
            thread.IsValid(),
            "Make sure there is a thread 0x333333333 after we load the python OS plug-in",
        )

        # Verify that the correct thread is selected
        thread = process.GetSelectedThread()
        self.assertEqual(thread.GetThreadID(), 0x111111111)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        # Test non-type template parameter packs.
        self.expect_expr(
            "myC",
            result_type="C<int, 16, 32>",
            result_children=[
                ValueCheck(
                    name="C<int, 16>", children=[ValueCheck(name="member", value="64")]
                )
            ],
        )
        self.expect_expr("myLesserC.argsAre_16_32()", result_value="false")
        self.expect_expr("myC.argsAre_16_32()", result_value="true")

        # Test type template parameter packs.
        self.expect_expr(
            "myD",
            result_type="D<int, int, bool>",
            result_children=[
                ValueCheck(
                    name="D<int, int>", children=[ValueCheck(name="member", value="64")]
                )
            ],
        )
        self.expect_expr("myLesserD.argsAre_Int_bool()", result_value="false")
        self.expect_expr("myD.argsAre_Int_bool()", result_value="true")

        # Disabling until we do template lookup correctly: http://lists.llvm.org/pipermail/lldb-commits/Week-of-Mon-20180507/040689.html
        # FIXME: Rewrite this with expect_expr
        # self.expect("expression -- C<int, 16>().isSixteenThirtyTwo()", DATA_TYPES_DISPLAYED_CORRECTLY, substrs = ["false"])
        # self.expect("expression -- C<int, 16, 32>().isSixteenThirtyTwo()", DATA_TYPES_DISPLAYED_CORRECTLY, substrs = ["true"])
        # self.expect("expression -- D<int, int>().isIntBool()", DATA_TYPES_DISPLAYED_CORRECTLY, substrs = ["false"])
        # self.expect("expression -- D<int, int, bool>().isIntBool()", DATA_TYPES_DISPLAYED_CORRECTLY, substrs = ["true"])



"""Test the LLDB module cache funcionality."""

import glob
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import os
import time


class ModuleCacheTestcaseBSD(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number in a(int) to break at.
        self.line_a = line_number("a.c", "// Set file and line breakpoint inside a().")
        self.line_b = line_number("b.c", "// Set file and line breakpoint inside b().")
        self.line_c = line_number("c.c", "// Set file and line breakpoint inside c().")
        self.cache_dir = os.path.join(self.getBuildDir(), "lldb-module-cache")
        # Set the lldb module cache directory to a directory inside the build
        # artifacts directory so no other tests are interfered with.
        self.runCmd(
            'settings set symbols.lldb-index-cache-path "%s"' % (self.cache_dir)
        )
        self.runCmd("settings set symbols.enable-lldb-index-cache true")
        self.build()

    def get_module_cache_files(self, basename):
        module_cache_glob = os.path.join(
            self.cache_dir, "llvmcache-*%s*symtab*" % (basename)
        )
        return glob.glob(module_cache_glob)

    # Requires no dSYM, so we let the Makefile make the right stuff for us
    @no_debug_info_test
    @skipUnlessDarwin
    def test(self):
        """
        This test has been modified to make sure .o files that don't have
        UUIDs are not cached after discovering build systems that play with
        modification times of .o files that the modification times are not
        unique enough to ensure the .o file within the .a file are the right
        files as this was causing older cache files to be accepted for new
        updated .o files.

        ELF .o files do calculate a UUID from the contents of the file,
        which is expensive, but no one loads .o files into a debug sessions
        when using ELF files. Mach-o .o files do not have UUID values and do
        no calculate one as they _are_ used during debug sessions when no
        dSYM file is generated. If we can find a way to uniquely and cheaply
        create UUID values for mach-o .o files in the future, this test will
        be updated to test this functionality. This test will now make sure
        there are no cache entries for any .o files in BSD archives.

        The old test case description is below in case we enable caching for
        .o files again:

        Test module cache functionality for bsd archive object files.

        This will test that if we enable the module cache, we have a
        corresponding cache entry for the .o files in libfoo.a.

        The static library has two entries for "a.o":
        - one from a.c
        - one from c.c which had c.o renamed to a.o and then put into the
          libfoo.a as an extra .o file with different contents from the
          original a.o

        We do this to test that we can correctly cache duplicate .o files
        that appear in .a files.

        This test only works on darwin because of the way DWARF is stored
        where the debug map will refer to .o files inside of .a files.
        """
        exe = self.getBuildArtifact("a.out")

        # Create a module with no dependencies.
        target = self.createTestTarget(load_dependent_modules=False)

        self.runCmd("breakpoint set -f a.c -l %d" % (self.line_a))
        self.runCmd("breakpoint set -f b.c -l %d" % (self.line_b))
        self.runCmd("breakpoint set -f c.c -l %d" % (self.line_c))

        # Get the executable module and get the number of symbols to make
        # sure the symbol table gets parsed and cached. The module cache is
        # enabled in the setUp() function.
        main_module = target.GetModuleAtIndex(0)
        self.assertTrue(main_module.IsValid())
        # Make sure the symbol table gets loaded and cached
        main_module.GetNumSymbols()
        a_o_cache_files = self.get_module_cache_files("libfoo.a(a.o)")
        b_o_cache_files = self.get_module_cache_files("libfoo.a(b.o)")

        # We expect the directory for a.o to have two cache directories:
        # - 1 for the a.o with a earlier mod time
        # - 1 for the a.o that was renamed from c.o that should be 2 seconds older
        # self.assertEqual(len(a_o_cache_files), 2,
        #         "make sure there are two files in the module cache directory (%s) for libfoo.a(a.o)" % (self.cache_dir))
        # self.assertEqual(len(b_o_cache_files), 1,
        #         "make sure there are two files in the module cache directory (%s) for libfoo.a(b.o)" % (self.cache_dir))

        # We are no longer caching .o files in the lldb index cache. If we ever
        # re-enable this functionality, we can uncomment out the above lines of
        # code.
        self.assertEqual(
            len(a_o_cache_files),
            0,
            "make sure there are no files in the module cache directory (%s) for libfoo.a(a.o)"
            % (self.cache_dir),
        )
        self.assertEqual(
            len(b_o_cache_files),
            0,
            "make sure there are no files in the module cache directory (%s) for libfoo.a(b.o)"
            % (self.cache_dir),
        )



# RUN: %PYTHON %s | FileCheck %s

import gc
from mlir.ir import *


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0
    return f


# CHECK-LABEL: TEST: testDialectDescriptor
@run
def testDialectDescriptor():
    ctx = Context()
    d = ctx.get_dialect_descriptor("func")
    # CHECK: <DialectDescriptor func>
    print(d)
    # CHECK: func
    print(d.namespace)
    try:
        _ = ctx.get_dialect_descriptor("not_existing")
    except ValueError:
        pass
    else:
        assert False, "Expected exception"


# CHECK-LABEL: TEST: testUserDialectClass
@run
def testUserDialectClass():
    ctx = Context()
    # Access using attribute.
    d = ctx.dialects.func
    # CHECK: <Dialect func (class mlir.dialects._func_ops_gen._Dialect)>
    print(d)
    try:
        _ = ctx.dialects.not_existing
    except AttributeError:
        pass
    else:
        assert False, "Expected exception"

    # Access using index.
    d = ctx.dialects["func"]
    # CHECK: <Dialect func (class mlir.dialects._func_ops_gen._Dialect)>
    print(d)
    try:
        _ = ctx.dialects["not_existing"]
    except IndexError:
        pass
    else:
        assert False, "Expected exception"

    # Using the 'd' alias.
    d = ctx.d["func"]
    # CHECK: <Dialect func (class mlir.dialects._func_ops_gen._Dialect)>
    print(d)


# CHECK-LABEL: TEST: testCustomOpView
# This test uses the standard dialect AddFOp as an example of a user op.
# TODO: Op creation and access is still quite verbose: simplify this test as
# additional capabilities come online.
@run
def testCustomOpView():
    def createInput():
        op = Operation.create("pytest_dummy.intinput", results=[f32])
        # TODO: Auto result cast from operation
        return op.results[0]

    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        m = Module.create()

        with InsertionPoint(m.body):
            f32 = F32Type.get()
            # Create via dialects context collection.
            input1 = createInput()
            input2 = createInput()
            op1 = ctx.dialects.arith.AddFOp(input1, input2)

            # Create via an import
            from mlir.dialects.arith import AddFOp

            AddFOp(input1, op1.result)

    # CHECK: %[[INPUT0:.*]] = "pytest_dummy.intinput"
    # CHECK: %[[INPUT1:.*]] = "pytest_dummy.intinput"
    # CHECK: %[[R0:.*]] = arith.addf %[[INPUT0]], %[[INPUT1]] : f32
    # CHECK: %[[R1:.*]] = arith.addf %[[INPUT0]], %[[R0]] : f32
    m.operation.print()


# CHECK-LABEL: TEST: testIsRegisteredOperation
@run
def testIsRegisteredOperation():
    ctx = Context()

    # CHECK: cf.cond_br: True
    print(f"cf.cond_br: {ctx.is_registered_operation('cf.cond_br')}")
    # CHECK: func.not_existing: False
    print(f"func.not_existing: {ctx.is_registered_operation('func.not_existing')}")



# TestSwiftGenericTupleLabels.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that LLDB can reconstruct tuple labels from metadata
"""
import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftGenericTupleLabels(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    def setUp(self):
        lldbtest.TestBase.setUp(self)

    @swiftTest
    def test_generic_tuple_labels(self):
        """Test that LLDB can reconstruct tuple labels from metadata"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        the_tuple = self.frame().FindVariable('x')
        the_tuple.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        the_tuple.SetPreferSyntheticValue(True)

        self.assertTrue(the_tuple.GetChildAtIndex(
            0).GetName() == 'x', '.0 == x')
        self.assertTrue(the_tuple.GetChildAtIndex(
            1).GetName() == '1', '.1 == 1')
        self.assertTrue(the_tuple.GetChildAtIndex(
            2).GetName() == 'z', '.2 == z')
        self.assertTrue(the_tuple.GetChildAtIndex(
            3).GetName() == '3', '.3 == 3')
        self.assertTrue(the_tuple.GetChildAtIndex(
            4).GetName() == 'q', '.4 == q')
        self.assertTrue(the_tuple.GetChildAtIndex(
            5).GetName() == 'w', '.5 == q')

        self.expect('frame variable -d run -- x.w', substrs=['72'])
        self.expect('expression -d run -- x.z', substrs=['36'])




import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil

class TestSwiftPrivateImport(TestBase):
    @skipUnlessDarwin
    @swiftTest
    def test_private_import(self):
        """Test a library with a private import for which there is no debug info"""
        invisible_swift = self.getBuildArtifact("Invisible.swift")
        import shutil
        shutil.copyfile("InvisibleSource.swift", invisible_swift)
        self.build()
        os.unlink(invisible_swift)
        os.unlink(self.getBuildArtifact("Invisible.swiftmodule"))
        os.unlink(self.getBuildArtifact("Invisible.swiftinterface"))

        log = self.getBuildArtifact("types.log")
        self.expect('log enable lldb types -f "%s"' % log)
        lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'),
            extra_images=['Library', 'Invisible'])

        precise = self.dbg.GetSetting('symbols.swift-precise-compiler-invocation').GetBooleanValue()
        if precise:
            # Test that importing the expression context (i.e., the module
            # "a") pulls in the explicit dependencies, but not their
            # private imports.  This comes before the other checks,
            # because type reconstruction will still trigger an import of
            # the "Invisible" module that we can't prevent later one.
            self.expect("expression 1+1")
            self.filecheck('platform shell cat "%s"' % log, __file__)
#           CHECK:  Module import remark: loaded module 'Library'
#           CHECK-NOT: Module import remark: loaded module 'Invisible'

        self.expect("fr var -d run -- x", substrs=["(Invisible.InvisibleStruct)"])
        self.expect("fr var -d run -- y", substrs=["(Library.Conforming)",
                                                   "conforming"])



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import mlir.dialects.gpu
import mlir.dialects.gpu.passes
from mlir.passmanager import *


def run(f):
    print("\nTEST:", f.__name__)
    f()


def testGPUPass():
    with Context() as context:
        PassManager.parse("any(gpu-kernel-outlining)")
    print("SUCCESS")


# CHECK-LABEL: testGPUPass
#       CHECK: SUCCESS
run(testGPUPass)



"""
Test that our use of "Optional" to wrap expressions in a weak self
context doesn't collide with names inherited from ObjC.
"""

from __future__ import print_function

import os
import time
import re
import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestOptionalAmbiguity(TestBase):
    @swiftTest
    def test_sample_rename_this(self):
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.swift")
        self.print_in_closure()

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

    def print_in_closure(self):
        (_, _, thread, _) = lldbutil.run_to_source_breakpoint(self,
                                "Set a breakpoint here", self.main_source_file) 

        # Since the failure mode of a name collision is that the expression fails
        # to compile, I just check that here.
        frame = thread.GetFrameAtIndex(0)
        ret_value = frame.EvaluateExpression("self")
        self.assertSuccess(ret_value.GetError(), "The expression completed successfully")





"""
Test that Swift types are displayed correctly in C++
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftBackwardInteropFormatSwiftTypesInCxx(TestBase):

    def setup(self, bkpt_str):
        self.build()
        
        _, _, _, _= lldbutil.run_to_source_breakpoint(
            self, bkpt_str, lldb.SBFileSpec('main.cpp'))

    @swiftTest
    def test_class(self):
        self.setup('Break here for class')

        self.expect('v swiftClass', substrs=['SwiftClass', 'field = 42', 
            'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
            '[3] = "strings"'])
        self.expect('expr swiftClass', substrs=['SwiftClass', 'field = 42', 
            'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
            '[3] = "strings"'])
        
    @swiftTest
    def test_subclass(self):
        self.setup('Break here for subclass')

        self.expect('v swiftSublass', substrs=['SwiftSubclass', 'field = 42', 
            'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
            '[3] = "strings"', 'extraField = "this is an extra subclass field"'])
        self.expect('expr swiftSublass', substrs=['SwiftSubclass', 'field = 42', 
            'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
            '[3] = "strings"', 'extraField = "this is an extra subclass field"'])

    @swiftTest
    def test_struct(self):
        self.setup('Break here for struct')

        self.expect('v swiftStruct', substrs=['SwiftStruct', 'str = "Hello this is a big string"', 
            'boolean = true'])
        self.expect('expr swiftStruct', substrs=['SwiftStruct', 'str = "Hello this is a big string"', 
            'boolean = true'])

    @swiftTest
    def test_generic_struct(self):
        self.setup('Break here for generic struct')

        self.expect('v wrapper', substrs=['a.GenericStructPair<a.SwiftClass, a.SwiftStruct>', 
                'field = 42', 'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
                '[3] = "strings"', 'str = "Hello this is a big string"', 'boolean = true'])
        self.expect('expr wrapper', substrs=['a.GenericStructPair<a.SwiftClass, a.SwiftStruct>', 
                'field = 42', 'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
                '[3] = "strings"', 'str = "Hello this is a big string"', 'boolean = true'])


    @swiftTest
    def test_generic_enum(self):
        self.setup('Break here for generic enum')

        self.expect('v swiftEnum', substrs=['a.GenericEnum<a.SwiftClass>', 'some',
                'field = 42', 'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
                '[3] = "strings"'])
        self.expect('expr swiftEnum', substrs=['a.GenericEnum<a.SwiftClass>', 'some',
                'field = 42', 'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
                '[3] = "strings"'])


    @swiftTest
    def test_swift_ivars(self):
        self.setup('Break here for swift ivars')

        self.expect('v type_with_ivars', substrs=['TypeWithSwiftIvars', 'swiftClass',
                'field = 42', 'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
                '[3] = "strings"', 'swiftSubclass', 'field = 42', 'arr = 4 values', 
                '[0] = "An"', '[1] = "array"', '[2] = "of"', '[3] = "strings"', 
                'extraField = "this is an extra subclass field"',
                'str = "Hello this is a big string"', 'boolean = true'])
        self.expect('expr type_with_ivars', substrs=['TypeWithSwiftIvars', 'swiftClass',
                'field = 42', 'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
                '[3] = "strings"', 'swiftSubclass', 'field = 42', 'arr = 4 values', 
                '[0] = "An"', '[1] = "array"', '[2] = "of"', '[3] = "strings"', 
                'extraField = "this is an extra subclass field"',
                'str = "Hello this is a big string"', 'boolean = true'])

    @swiftTest
    def test_typealias(self):
        self.setup('Break here for swift alias')

        self.expect('v aliased', substrs=['SwiftClass', 'field = 42', 
            'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
            '[3] = "strings"'])
        self.expect('expr aliased', substrs=['SwiftClass', 'field = 42', 
            'arr = 4 values', '[0] = "An"', '[1] = "array"', '[2] = "of"', 
            '[3] = "strings"'])



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.IsValid()
    obj.GetName()
    obj.SetEnabled(True)
    obj.IsEnabled()
    obj.SetOneShot(True)
    obj.IsOneShot()
    obj.SetIgnoreCount(1)
    obj.GetIgnoreCount()
    obj.SetCondition("1 == 2")
    obj.GetCondition()
    obj.SetAutoContinue(False)
    obj.GetAutoContinue()
    obj.SetThreadID(0x1234)
    obj.GetThreadID()
    obj.SetThreadIndex(10)
    obj.GetThreadIndex()
    obj.SetThreadName("AThread")
    obj.GetThreadName()
    obj.SetQueueName("AQueue")
    obj.GetQueueName()
    obj.SetScriptCallbackFunction("AFunction")
    commands = lldb.SBStringList()
    obj.SetCommandLineCommands(commands)
    obj.GetCommandLineCommands(commands)
    obj.SetScriptCallbackBody("Insert Python Code here")
    obj.GetAllowList()
    obj.SetAllowList(False)
    obj.GetAllowDelete()
    obj.SetAllowDelete(False)
    obj.GetAllowDisable()
    obj.SetAllowDisable(False)
    stream = lldb.SBStream()
    obj.GetDescription(stream)



"""
Test that the built in ObjC exception throw recognizer works
"""

import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestObjCRecognizer(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessDarwin
    def test_exception_recognizer_sub_class(self):
        """There can be many tests in a test case - describe this test here."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.m")
        self.objc_recognizer_test(True)

    @skipUnlessDarwin
    def test_exception_recognizer_plain(self):
        """There can be many tests in a test case - describe this test here."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.m")
        self.objc_recognizer_test(False)

    def objc_recognizer_test(self, sub_class):
        """Make sure we stop at the exception and get all the fields out of the recognizer.
        If sub_class is True, we make a subclass of NSException and throw that."""
        if sub_class:
            bkpt_string = "Set a breakpoint here for MyException"
        else:
            bkpt_string = "Set a breakpoint here for plain exception"

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, bkpt_string, self.main_source_file
        )

        # Now turn on the ObjC Exception breakpoint and continue to hit it:
        exception_bkpt = target.BreakpointCreateForException(
            lldb.eLanguageTypeObjC, False, True
        )
        self.assertTrue(
            exception_bkpt.GetNumLocations() > 0, "Got some exception locations"
        )

        threads = lldbutil.continue_to_breakpoint(process, exception_bkpt)
        self.assertEqual(len(threads), 1, "One thread hit exception breakpoint")
        frame = threads[0].frame[0]

        var_opts = lldb.SBVariablesOptions()
        var_opts.SetIncludeRecognizedArguments(True)
        var_opts.SetUseDynamic(True)
        vars = frame.GetVariables(var_opts)
        self.assertEqual(len(vars), 1, "Got the synthetic argument")
        self.assertTrue(vars[0].IsValid(), "Got a valid Exception variable")

        # This will be a pointer

        ns_exception_children = [
            ValueCheck(type="NSObject"),
            ValueCheck(name="name", summary='"NSException"'),
            ValueCheck(name="reason", summary='"Simple Reason"'),
            ValueCheck(name="userInfo"),
            ValueCheck(name="reserved"),
        ]
        ns_exception = ValueCheck(type="NSException", children=ns_exception_children)
        if not sub_class:
            simple_check = ValueCheck(name="exception", dereference=ns_exception)
            simple_check.check_value(self, vars[0], "Simple exception is right")
        else:
            my_exception_children = [
                ns_exception,
                ValueCheck(name="extra_info", type="int", value="100"),
            ]
            my_exception = ValueCheck(
                type="MyException", children=my_exception_children
            )
            sub_check = ValueCheck(
                name="exception", type="MyException *", dereference=my_exception
            )
            sub_check.check_value(self, vars[0], "Subclass exception is right")



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftDWARFImporter_Swift(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    def build(self):
        include = self.getBuildArtifact('include')
        inputs = self.getSourcePath(os.path.join('Inputs', 'Modules'))
        lldbutil.mkdir_p(include)
        import shutil
        for f in ['module.modulemap', 'objc-header.h']:
            shutil.copyfile(os.path.join(inputs, f), os.path.join(include, f))

        super(TestSwiftDWARFImporter_Swift, self).build()

        # Remove the header files to thwart ClangImporter.
        self.assertTrue(os.path.isdir(include))
        shutil.rmtree(include)

    @skipUnlessDarwin
    @swiftTest
    def test(self):
        self.runCmd("settings set symbols.use-swift-dwarfimporter true")
        self.build()
        dylib = self.getBuildArtifact('libLibrary.dylib')
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'),
            extra_images = [dylib])

        log = self.getBuildArtifact("types.log")
        self.runCmd('log enable lldb types -f "%s"' % log)

        self.expect("target var -d run myobj", substrs=["(ObjCClass)"])

        found = 0
        response = 0
        import io
        logfile = io.open(log, "r", encoding='utf-8')
        for line in logfile:
            if 'SwiftDWARFImporterDelegate::lookupValue("ObjCClass")' in line:
                found += 1
            elif found == 1 and response == 0 and 'SwiftDWARFImporterDelegate' in line:
                self.assertTrue('from debug info' in line, line)
                response += 1
            elif found == 2 and response == 1 and 'SwiftDWARFImporterDelegate' in line:
                self.assertTrue('types collected' in line, line)
                response += 1
        self.assertEqual(found, 1)
        self.assertEqual(response, 1)



#!/usr/bin/env python
"""Inferior program used by process control tests."""

import argparse
import datetime
import signal
import subprocess
import sys
import time


def parse_args(command_line):
    """Parses the command line arguments given to it.

    @param command_line a list of command line arguments to be parsed.

    @return the argparse options dictionary.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--ignore-signal",
        "-i",
        dest="ignore_signals",
        metavar="SIGNUM",
        action="append",
        type=int,
        default=[],
        help="ignore the given signal number (if possible)",
    )
    parser.add_argument(
        "--launch-child-share-handles",
        action="store_true",
        help=(
            "launch a child inferior.py that shares stdout/stderr/stdio and "
            "never returns"
        ),
    )
    parser.add_argument(
        "--never-return",
        action="store_true",
        help="run in an infinite loop, never return",
    )
    parser.add_argument(
        "--return-code",
        "-r",
        type=int,
        default=0,
        help="specify the return code for the inferior upon exit",
    )
    parser.add_argument(
        "--sleep",
        "-s",
        metavar="SECONDS",
        dest="sleep_seconds",
        type=float,
        help="sleep for SECONDS seconds before returning",
    )
    parser.add_argument(
        "--verbose",
        "-v",
        action="store_true",
        help="log verbose operation details to stdout",
    )
    return parser.parse_args(command_line)


def handle_ignore_signals(options, signals):
    """Ignores any signals provided to it.

    @param options the command line options parsed by the program.
    General used to check flags for things like verbosity.

    @param signals the list of signals to ignore.  Can be None or zero-length.
    Entries should be type int.
    """
    if signals is None:
        return

    for signum in signals:
        if options.verbose:
            print("disabling signum {}".format(signum))
        signal.signal(signum, signal.SIG_IGN)


def handle_sleep(options, sleep_seconds):
    """Sleeps the number of seconds specified, restarting as needed.

    @param options the command line options parsed by the program.
    General used to check flags for things like verbosity.

    @param sleep_seconds the number of seconds to sleep.  If None
    or <= 0, no sleeping will occur.
    """
    if sleep_seconds is None:
        return

    if sleep_seconds <= 0:
        return

    end_time = datetime.datetime.now() + datetime.timedelta(0, sleep_seconds)
    if options.verbose:
        print("sleep end time: {}".format(end_time))

    # Do sleep in a loop: signals can interrupt.
    while datetime.datetime.now() < end_time:
        # We'll wrap this in a try/catch so we don't encounter
        # a race if a signal (ignored) knocks us out of this
        # loop and causes us to return.
        try:
            sleep_interval = end_time - datetime.datetime.now()
            sleep_seconds = sleep_interval.total_seconds()
            if sleep_seconds > 0:
                time.sleep(sleep_seconds)
        except:  # pylint: disable=bare-except
            pass


def handle_launch_children(options):
    if options.launch_child_share_handles:
        # Launch the child, share our file handles.
        # We won't bother reaping it since it will likely outlive us.
        subprocess.Popen([sys.executable, __file__, "--never-return"])


def handle_never_return(options):
    if not options.never_return:
        return

    # Loop forever.
    while True:
        try:
            time.sleep(10)
        except:  # pylint: disable=bare-except
            # Ignore
            pass


def main(command_line):
    """Drives the main operation of the inferior test program.

    @param command_line the command line options to process.

    @return the exit value (program return code) for the process.
    """
    options = parse_args(command_line)
    handle_ignore_signals(options, options.ignore_signals)
    handle_launch_children(options)
    handle_sleep(options, options.sleep_seconds)
    handle_never_return(options)

    return options.return_code


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



"""
Test basic std::array functionality.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.import-std-module true")

        size_type = "size_type"
        value_type = "value_type"
        iterator = "iterator"
        riterator = "reverse_iterator"

        # Test inspecting an array of integers.
        array_type = "std::array<int, 3>"
        self.expect_expr(
            "a",
            result_type=array_type,
            result_children=[
                ValueCheck(
                    name="__elems_",
                    children=[
                        ValueCheck(value="3"),
                        ValueCheck(value="1"),
                        ValueCheck(value="2"),
                    ],
                )
            ],
        )
        self.expect_expr("a.size()", result_type=size_type, result_value="3")
        self.expect_expr("a.front()", result_type=value_type, result_value="3")
        self.expect_expr("a[1]", result_type=value_type, result_value="1")
        self.expect_expr("a.back()", result_type=value_type, result_value="2")

        # Both are just pointers to the underlying elements.
        self.expect_expr("a.begin()", result_type=iterator)
        self.expect_expr("a.rbegin()", result_type=riterator)

        self.expect_expr("*a.begin()", result_type=value_type, result_value="3")
        self.expect_expr("*a.rbegin()", result_type="int", result_value="2")

        self.expect_expr("a.at(0)", result_type=value_type, result_value="3")

        # Same again with an array that has an element type from debug info.
        array_type = "std::array<DbgInfo, 1>"

        dbg_info_elem_children = [ValueCheck(value="4")]
        dbg_info_elem = [ValueCheck(children=dbg_info_elem_children)]

        self.expect_expr(
            "b",
            result_type=array_type,
            result_children=[ValueCheck(name="__elems_", children=dbg_info_elem)],
        )
        self.expect_expr("b.size()", result_type=size_type, result_value="1")
        self.expect_expr(
            "b.front()", result_type=value_type, result_children=dbg_info_elem_children
        )
        self.expect_expr(
            "b[0]", result_type=value_type, result_children=dbg_info_elem_children
        )
        self.expect_expr(
            "b.back()", result_type=value_type, result_children=dbg_info_elem_children
        )

        # Both are just pointers to the underlying elements.
        self.expect_expr("b.begin()", result_type=iterator)
        self.expect_expr("b.rbegin()", result_type=riterator)

        self.expect_expr(
            "*b.begin()", result_type=value_type, result_children=dbg_info_elem_children
        )
        self.expect_expr(
            "*b.rbegin()", result_type="DbgInfo", result_children=dbg_info_elem_children
        )

        self.expect_expr(
            "b.at(0)", result_type=value_type, result_children=dbg_info_elem_children
        )



# TestSwiftAssociatedTypeResolution.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that associated-typed objects get resolved to their proper location in memory
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftArchetypeResolution(TestBase):
    @swiftTest
    def test_swift_associated_type_resolution(self):
        """Test that archetype-typed objects get resolved to their proper location in memory"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'Set a breakpoint here', lldb.SBFileSpec('main.swift'))

        var = self.frame().FindVariable("things")
        var.SetPreferDynamicValue(lldb.eDynamicCanRunTarget)
        var.SetPreferSyntheticValue(True)
        self.assertSuccess(var.GetError(), "Failed to get things")
        self.assertEqual(var.GetNumChildren(), 4,
                         "Got the right number of children")
        type_name = var.GetTypeName()
        self.assertEqual(type_name, "Swift.Array<Swift.Int>",
                         "Wrong typename: %s."%(type_name))
        for i in range(0,4):
            child = var.GetChildAtIndex(i)
            self.assertSuccess(child.GetError(), "Failed to get things[%d]" % i)
            value = child.GetValueAsUnsigned()
            self.assertEqual(value, i, "Wrong value: %d not %d."%(value, i))



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftExpressionLanguageVersion(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @swiftTest
    def test(self):
        """Test changing the Swift language version"""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_name_breakpoint(
            self, 'main')

        expr = """\
#if swift(>=6.0)
6
#else
5
#endif
        """

        def test_version(n):
            if self.TraceOn():
                print("Testing version %d"%n)
            options = lldb.SBExpressionOptions()
            options.SetLanguage(lldb.eLanguageNameSwift, n*100 + 0)
            value = self.frame().EvaluateExpression(expr, options)
            self.assertEquals(value.GetValue(), "%d" % n)

        test_version(5)
        test_version(6)



# RUN: %PYTHON %s

import doctest
import importlib


def test_module(module_name):
    print(f"--- Testing module: {module_name}")
    m = importlib.import_module(module_name)
    doctest.testmod(m, verbose=True, raise_on_error=True, report=True)


test_module("mlir.dialects.linalg.opdsl.lang.affine")
test_module("mlir.dialects.linalg.opdsl.lang.types")



# -*- Python -*-

# Configuration file for the 'lit' test runner.

import os
import subprocess

import lit.formats

# name: The name of this test suite.
config.name = "MLIR-Unit"

# suffixes: A list of file extensions to treat as test files.
config.suffixes = []

# test_source_root: The root path where tests are located.
# test_exec_root: The root path where tests should be run.
config.test_exec_root = os.path.join(config.mlir_obj_root, "unittests")
config.test_source_root = config.test_exec_root

# testFormat: The test format to use to interpret tests.
config.test_format = lit.formats.GoogleTest(config.llvm_build_mode, "Tests")

# Propagate the temp directory. Windows requires this because it uses \Windows\
# if none of these are present.
if "TMP" in os.environ:
    config.environment["TMP"] = os.environ["TMP"]
if "TEMP" in os.environ:
    config.environment["TEMP"] = os.environ["TEMP"]

# Propagate HOME as it can be used to override incorrect homedir in passwd
# that causes the tests to fail.
if "HOME" in os.environ:
    config.environment["HOME"] = os.environ["HOME"]

# Propagate sanitizer options.
for var in [
    "ASAN_SYMBOLIZER_PATH",
    "HWASAN_SYMBOLIZER_PATH",
    "MSAN_SYMBOLIZER_PATH",
    "TSAN_SYMBOLIZER_PATH",
    "UBSAN_SYMBOLIZER_PATH",
    "ASAN_OPTIONS",
    "HWASAN_OPTIONS",
    "MSAN_OPTIONS",
    "TSAN_OPTIONS",
    "UBSAN_OPTIONS",
]:
    if var in os.environ:
        config.environment[var] = os.environ[var]



"""
Test that LLDB can launch a linux executable and then execs into the dynamic
loader into this program again.
"""

import lldb
import os

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestLinux64ExecViaDynamicLoader(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfXmlSupportMissing
    @skipIf(oslist=no_match(["linux"]))
    def test_with_svr4(self):
        self.runCmd("settings set plugin.process.gdb-remote.use-libraries-svr4 true")
        self._test()

    @skipIf(oslist=no_match(["linux"]))
    def test_without_svr4(self):
        self.runCmd("settings set plugin.process.gdb-remote.use-libraries-svr4 false")
        self._test()

    def _test(self):
        self.build()

        # Extracts path of the interpreter.
        exe = self.getBuildArtifact("a.out")

        spec = lldb.SBModuleSpec()
        spec.SetFileSpec(lldb.SBFileSpec(exe))
        interp_section = lldb.SBModule(spec).FindSection(".interp")
        if not interp_section:
            return
        section_data = interp_section.GetSectionData()
        error = lldb.SBError()
        dyld_path = section_data.GetString(error, 0)
        if error.Fail():
            return

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set a breakpoint in the main function that will get hit after the
        # program exec's via the dynamic loader. The breakpoint will only get
        # hit if we can successfully read the shared library lists in the
        # DynamicLoaderPOSIXDYLD.cpp when we exec into the dynamic loader.
        breakpoint_main = target.BreakpointCreateBySourceRegex(
            "// Break here", lldb.SBFileSpec("main.cpp")
        )
        # Setup our launch info to supply the dynamic loader path to the
        # program so it gets two args:
        # - path to a.out
        # - path to dynamic loader
        launch_info = lldb.SBLaunchInfo([dyld_path])
        error = lldb.SBError()
        process = target.Launch(launch_info, error)
        self.assertSuccess(error)

        threads = lldbutil.get_stopped_threads(process, lldb.eStopReasonExec)
        self.assertEqual(len(threads), 1, "We got a thread stopped for exec.")

        process.Continue()

        # Stopped on main here.
        self.assertState(process.GetState(), lldb.eStateStopped)
        thread = process.GetSelectedThread()
        self.assertIn("main", thread.GetFrameAtIndex(0).GetDisplayFunctionName())



import lldbsuite.test.lldbinline as lldbinline
from lldbsuite.test.decorators import *

# FIXME! The Swift driver insists on passing -experimental-skip-non-inlinable-function-bodies-without-types to -emit-module.
lldbinline.MakeInlineTest(__file__, globals(), decorators=[swiftTest, skipIf(bugnumber="rdar://120928396")])



"""
Tests that C strings work as expected in expressions
"""
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CStringsTestCase(TestBase):
    def test_with_run_command(self):
        """Tests that C strings work as expected in expressions"""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        line = line_number("main.c", "// breakpoint 1")
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("process launch", RUN_SUCCEEDED)

        self.expect("expression -- a[2]", patterns=["\((const )?char\) \$0 = 'c'"])

        self.expect("expression -- z[2]", startstr="(const char) $1 = 'x'")

        # On Linux, the expression below will test GNU indirect function calls.
        self.expect('expression -- (int)strlen("hello")', startstr="(int) $2 = 5")

        self.expect('expression -- "world"[2]', startstr="(const char) $3 = 'r'")

        self.expect('expression -- ""[0]', startstr="(const char) $4 = '\\0'")

        self.expect('expr --raw -- "hello"', substrs=["[0] = 'h'", "[5] = '\\0'"])

        self.expect('expression "hello"', substrs=["[6]) $", "hello"])

        self.expect(
            'expression (char*)"hello"', substrs=["(char *) $", " = 0x", "hello"]
        )

        self.expect('expression (int)strlen("")', substrs=["(int) $", " = 0"])

        self.expect("expression !z", substrs=["false"])



"""
Test that lldb command "command source" works correctly.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CommandSourceTestCase(TestBase):
    @no_debug_info_test
    def test_command_source(self):
        """Test that lldb command "command source" works correctly."""

        # Sourcing .lldb in the current working directory, which in turn imports
        # the "my" package that defines the date() function.
        self.runCmd("command source .lldb")
        self.check_results()

    @no_debug_info_test
    def test_command_source_relative(self):
        """Test that lldb command "command source" works correctly with relative paths."""

        # Sourcing .lldb in the current working directory, which in turn imports
        # the "my" package that defines the date() function.
        self.runCmd("command source commands2.txt")
        self.check_results()

    def check_results(self, failure=False):
        # Python should evaluate "my.date()" successfully.
        command_interpreter = self.dbg.GetCommandInterpreter()
        self.assertTrue(command_interpreter, VALID_COMMAND_INTERPRETER)
        result = lldb.SBCommandReturnObject()
        command_interpreter.HandleCommand("script my.date()", result)

        import datetime

        if failure:
            self.expect(
                result.GetOutput(),
                "script my.date() runs successfully",
                exe=False,
                error=True,
            )
        else:
            self.expect(
                result.GetOutput(),
                "script my.date() runs successfully",
                exe=False,
                substrs=[str(datetime.date.today())],
            )

    @no_debug_info_test
    def test_command_source_relative_error(self):
        """Test that 'command source -C' gives an error for a relative path"""
        source_dir = self.getSourceDir()
        result = lldb.SBCommandReturnObject()
        self.runCmd("command source --stop-on-error 1 not-relative.txt")
        self.check_results(failure=True)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    @skipIfWindows
    def test_resize_no_editline(self):
        """Tests terminal resizing if the editline isn't used."""
        dbg = lldb.SBDebugger.Create(False)
        # Set the input handle to some stream so that we don't start the
        # editline interface.
        dbg.SetInputFileHandle(open("input_file"), True)
        opts = lldb.SBCommandInterpreterRunOptions()
        # Launch the command interpreter now.
        dbg.RunCommandInterpreter(True, True, opts, 0, False, False)
        # Try resizing the terminal which shouldn't crash.
        dbg.SetTerminalWidth(47)



"""Test the lldb public C++ api breakpoint callbacks."""

# __package__ = "lldbsuite.test"


import os
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import unittest2


@unittest2.skip("skipping due to frequent timeouts: rdar://28183131")
@skipIfNoSBHeaders
class SBBreakpointCallbackCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        self.generateSource("driver.cpp")
        self.generateSource("listener_test.cpp")
        self.generateSource("test_breakpoint_callback.cpp")
        self.generateSource("test_breakpoint_location_callback.cpp")
        self.generateSource("test_listener_event_description.cpp")
        self.generateSource("test_listener_event_process_state.cpp")
        self.generateSource("test_listener_resume.cpp")
        self.generateSource("test_stop-hook.cpp")

    @skipIfRemote
    # clang-cl does not support throw or catch (llvm.org/pr24538)
    @skipIfWindows
    def test_python_stop_hook(self):
        """Test that you can run a python command in a stop-hook when stdin is File based."""
        self.build_and_test("driver.cpp test_stop-hook.cpp", "test_python_stop_hook")

    @skipIfRemote
    # clang-cl does not support throw or catch (llvm.org/pr24538)
    @skipIfWindows
    def test_breakpoint_callback(self):
        """Test the that SBBreakpoint callback is invoked when a breakpoint is hit."""
        self.build_and_test(
            "driver.cpp test_breakpoint_callback.cpp", "test_breakpoint_callback"
        )

    @skipIfRemote
    # clang-cl does not support throw or catch (llvm.org/pr24538)
    @skipIfWindows
    def test_breakpoint_location_callback(self):
        """Test the that SBBreakpointLocation callback is invoked when a breakpoint is hit."""
        self.build_and_test(
            "driver.cpp test_breakpoint_location_callback.cpp",
            "test_breakpoint_location_callback",
        )

    @skipIfRemote
    # clang-cl does not support throw or catch (llvm.org/pr24538)
    @skipIfWindows
    @expectedFlakeyFreeBSD
    def test_sb_api_listener_event_description(self):
        """Test the description of an SBListener breakpoint event is valid."""
        self.build_and_test(
            "driver.cpp listener_test.cpp test_listener_event_description.cpp",
            "test_listener_event_description",
        )

    @skipIfRemote
    # clang-cl does not support throw or catch (llvm.org/pr24538)
    @skipIfWindows
    @expectedFlakeyFreeBSD
    def test_sb_api_listener_event_process_state(self):
        """Test that a registered SBListener receives events when a process
        changes state.
        """
        self.build_and_test(
            "driver.cpp listener_test.cpp test_listener_event_process_state.cpp",
            "test_listener_event_process_state",
        )

    @skipIfRemote
    # clang-cl does not support throw or catch (llvm.org/pr24538)
    @skipIfWindows
    @expectedFlakeyFreeBSD
    @skipIf(oslist=["linux"])  # flakey
    def test_sb_api_listener_resume(self):
        """Test that a process can be resumed from a non-main thread."""
        self.build_and_test(
            "driver.cpp listener_test.cpp test_listener_resume.cpp",
            "test_listener_resume",
        )

    def build_and_test(self, sources, test_name, args=None):
        """Build LLDB test from sources, and run expecting 0 exit code"""

        # These tests link against host lldb API.
        # Compiler's target triple must match liblldb triple
        # because remote is disabled, we can assume that the os is the same
        # still need to check architecture
        if self.getLldbArchitecture() != self.getArchitecture():
            self.skipTest(
                "This test is only run if the target arch is the same as the lldb binary arch"
            )

        self.inferior = "inferior_program"
        self.buildProgram("inferior.cpp", self.inferior)
        self.addTearDownHook(lambda: os.remove(self.getBuildArtifact(self.inferior)))

        self.buildDriver(sources, test_name)
        self.addTearDownHook(lambda: os.remove(self.getBuildArtifact(test_name)))

        test_exe = self.getBuildArtifact(test_name)
        exe = [test_exe, self.getBuildArtifact(self.inferior)]

        env = {self.dylibPath: self.getLLDBLibraryEnvVal()}
        if "LLDB_DEBUGSERVER_PATH" in os.environ:
            env["LLDB_DEBUGSERVER_PATH"] = os.environ["LLDB_DEBUGSERVER_PATH"]
        try:
            if self.TraceOn():
                print("Running test %s" % " ".join(exe))
                check_call(exe, env=env)
            else:
                with open(os.devnull, "w") as fnull:
                    check_call(exe, env=env, stdout=fnull, stderr=fnull)
        except subprocess.CalledProcessError as e:
            self.fail(e)

    def build_program(self, sources, program):
        return self.buildDriver(sources, program)



# TestSwiftUnknownReference.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2018 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftUnknownReference(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    def check_class(self, var_self):
        lldbutil.check_variable(self, var_self, num_children=2)
        m_base_string = var_self.GetChildMemberWithName("base_string")
        m_string = var_self.GetChildMemberWithName("string")
        lldbutil.check_variable(self, m_base_string, summary='"hello"')
        lldbutil.check_variable(self, m_string, summary='"world"')

    
    @swiftTest
    @skipUnlessFoundation
    def test_unknown_objc_ref(self):
        """Test unknown references to Objective-C objects."""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        frame = thread.frames[0]
        var_self = frame.FindVariable("self")
        m_pure_ref = var_self.GetChildMemberWithName("pure_ref")
        self.check_class(m_pure_ref)
        m_objc_ref = var_self.GetChildMemberWithName("objc_ref")
        self.check_class(m_objc_ref)




"""
Test that "memory region" command can show memory tagged regions
on AArch64 Linux.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AArch64LinuxMTEMemoryRegionTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    @skipUnlessAArch64MTELinuxCompiler
    def test_mte_regions(self):
        if not self.isAArch64MTE():
            self.skipTest("Target must support MTE.")

        # This test assumes that we have /proc/<PID>/smaps files
        # that include "VmFlags:" lines.
        # AArch64 kernel config defaults to enabling smaps with
        # PROC_PAGE_MONITOR and "VmFlags" was added in kernel 3.8,
        # before MTE was supported at all.

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Set break point at this line."),
            num_expected_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        substrs = ["memory tagging: enabled"]
        # The new page will be tagged
        self.expect("memory region the_page", substrs=substrs)
        # Code page will not be
        self.expect("memory region main", substrs=substrs, matching=False)



# TestSwiftInterfaceNoDebugInfo.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2019 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# -----------------------------------------------------------------------------
"""
Test that we load and handle swift modules that only have textual
.swiftinterface files -- i.e. no associated .swiftmodule file -- and no debug
info. The module loader should generate the .swiftmodule for any
.swiftinterface it finds unless it is already in the module cache.
"""

import glob
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import os.path
import unittest2


class TestSwiftInterfaceStaticNoDebugInfo(TestBase):
    @swiftTest
    def test_swift_interface(self):
        """Test that we load and handle modules that only have textual .swiftinterface files"""
        self.build()
        self.do_test()

    @swiftTest
    def test_swift_interface_fallback(self):
        """Test that we fall back to load from the .swiftinterface file if the .swiftmodule is invalid"""
        self.build()
        # install invalid modules in the build directory first to check we still fall back to the .swiftinterface
        modules = ['AA.swiftmodule', 'BB.swiftmodule', 'CC.swiftmodule']
        for module in modules:
            open(self.getBuildArtifact(module), 'w').close()
        self.do_test()

    def do_test(self):
        # The custom swift module cache location
        swift_mod_cache = self.getBuildArtifact("MCP")

        # Clear the swift module cache (populated by the Makefile build)
        shutil.rmtree(swift_mod_cache)
        self.assertFalse(os.path.isdir(swift_mod_cache),
                         "module cache should not exist")

        # Update the settings to use the custom module cache location.
        # Note: the clang module cache path setting is used for this currently.
        self.runCmd('settings set symbols.clang-modules-cache-path "%s"' % swift_mod_cache)

        # Set a breakpoint in and launch the main executable
        lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'),
            exe_name=self.getBuildArtifact("main"))

        # Check we are able to access the public fields of variables whose
        # types are from the .swiftinterface-only modules
        var = self.frame().FindVariable("x")
        lldbutil.check_variable(self, var, False, typename="AA.MyPoint")

        child_y = var.GetChildMemberWithName("y") # MyPoint.y is public
        lldbutil.check_variable(self, child_y, False, value="0")

        # MyPoint.x isn't public, but LLDB can find it through type metadata.
        child_x = var.GetChildMemberWithName("x")
        self.assertTrue(child_x.IsValid())

        # Expression evaluation using types from the .swiftinterface only
        # modules should work too
        lldbutil.check_expression(self, self.frame(),
                                  "y.magnitudeSquared", "404",
                                  use_summary=False)
        lldbutil.check_expression(self, self.frame(),
                                  "MyPoint(x: 1, y: 2).magnitudeSquared", "5",
                                  use_summary=False)

        # Check the swift module cache was populated with the .swiftmodule
        # files of the loaded modules
        self.assertTrue(os.path.isdir(swift_mod_cache), "module cache exists")
        a_modules = glob.glob(os.path.join(swift_mod_cache, 'AA-*.swiftmodule'))
        b_modules = glob.glob(os.path.join(swift_mod_cache, 'BB-*.swiftmodule'))
        c_modules = glob.glob(os.path.join(swift_mod_cache, 'CC-*.swiftmodule'))
        self.assertEqual(len(a_modules), 1)
        self.assertEqual(len(b_modules), 1)
        self.assertEqual(len(c_modules), 0)

        # Update the timestamps of the modules to a time well in the past
        for file in a_modules + b_modules:
            make_old(file)

        # Re-import module A and B
        self.runCmd("expr import AA")
        self.runCmd("expr import BB")

        # Import C for the first time
        self.runCmd("expr import CC")

        # Check we still have a single .swiftmodule in the cache for A and B
        # and that there is now one for C too
        a_modules = glob.glob(os.path.join(swift_mod_cache, 'AA-*.swiftmodule'))
        b_modules = glob.glob(os.path.join(swift_mod_cache, 'BB-*.swiftmodule'))
        c_modules = glob.glob(os.path.join(swift_mod_cache, 'CC-*.swiftmodule'))
        self.assertEqual(len(a_modules), 1, "unexpected number of swiftmodules for A.swift")
        self.assertEqual(len(b_modules), 1, "unexpected number of swiftmodules for B.swift")
        self.assertEqual(len(c_modules), 1, "unexpected number of swiftmodules for C.swift")

        # Make sure the .swiftmodule files of A and B were re-used rather than
        # re-generated when they were re-imported
        for file in a_modules + b_modules:
            self.assertTrue(is_old(file), "Swiftmodule file was regenerated rather than reused")


OLD_TIMESTAMP = 1390550700 # 2014-01-24T08:05:00+00:00

def make_old(file):
    """Sets the access and modified time of the given file to a time long past"""
    os.utime(file, (OLD_TIMESTAMP, OLD_TIMESTAMP))

def is_old(file):
    """Checks the modified time of the given file matches the timestamp set my make_old"""
    return os.stat(file).st_mtime == OLD_TIMESTAMP




# TestCGImportedTypes.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that we are able to deal with C-imported types (from CoreGraphics)
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftCGImportedTypes(TestBase):
    @skipUnlessDarwin
    @swiftTest
    def test_swift_cg_imported_types(self):
        """Test that we are able to deal with C-imported types from CoreGraphics"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        rect = self.frame().FindVariable("cgrect")
        self.assertTrue(rect.IsValid(), "Got the cgrect variable")
        origin_var = rect.GetChildMemberWithName("origin")
        self.assertTrue(origin_var.IsValid(), "Got origin from cgrect")
        x_var = origin_var.GetChildMemberWithName("x")
        self.assertTrue(x_var.IsValid(), "Got valid x from cgrect.origin")
        x_native = x_var.GetChildMemberWithName("native")
        self.assertTrue(
            x_native.IsValid(),
            "Got valid native from cgrect.origin.x")
        self.assertEqual(x_native.GetValue(), "10", "Value of x is correct")



"""
Test the ptr_refs tool on Darwin
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestPtrRefs(TestBase):
    @skipIfAsan  # The output looks different under ASAN.
    @skipUnlessDarwin
    def test_ptr_refs(self):
        """Test format string functionality."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        main_file_spec = lldb.SBFileSpec("main.c")
        breakpoint = target.BreakpointCreateBySourceRegex("break", main_file_spec)
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() == 1, VALID_BREAKPOINT
        )

        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        # Frame #0 should be on self.line1 and the break condition should hold.
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertTrue(
            thread.IsValid(),
            "There should be a thread stopped due to breakpoint condition",
        )

        frame = thread.GetFrameAtIndex(0)

        self.runCmd("command script import lldb.macosx.heap")
        self.expect("ptr_refs my_ptr", substrs=["malloc", "stack"])



# TestSwiftDedupMacros.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2018 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2

class TestSwiftDedupMacros(TestBase):
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(setting=('symbols.swift-precise-compiler-invocation', 'true'))
    # NOTE: rdar://44201206 - This test may sporadically segfault. It's likely
    # that the underlying memory corruption issue has been addressed, but due
    # to the difficulty of reproducing the crash, we are not sure. If a crash
    # is observed, try to collect a crashlog before disabling this test.
    @skipUnlessDarwin
    @swiftTest
    def testSwiftDebugMacros(self):
        """This tests that configuration macros get uniqued when building the
        scratch ast context. Note that "-D MACRO" options with a space
        are currently only combined to "-DMACRO" when they appear
        outside of the main binary.

        """
        self.build()
            
        target,  _, _, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec('dylib.swift'),
            extra_images=['Dylib'])

        # Turn on logging.
        log = self.getBuildArtifact("types.log")
        self.expect('log enable lldb types -f "%s"' % log)

        self.expect("expression foo", DATA_TYPES_DISPLAYED_CORRECTLY, substrs=["42"])
        self.filecheck('platform shell cat "%s"' % log, __file__)
#       CHECK: SwiftASTContextForExpressions{{.*}}-DDEBUG=1
#       CHECK: SwiftASTContextForExpressions{{.*}}-DSPACE
#       CHECK-NOT: {{ SPACE}}
#       CHECK: SwiftASTContextForExpressions{{.*}}-UNDEBUG
#       CHECK: SwiftASTContextForModule("libDylib{{.*}}-DDEBUG=1
#       CHECK: SwiftASTContextForModule("libDylib{{.*}}-DSPACE
#       CHECK-NOT: {{ SPACE}}
#       CHECK: SwiftASTContextForModule("libDylib{{.*}}-UNDEBUG



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil

class TestSwiftPrivateGenericType(TestBase):

    def setUp(self):
        TestBase.setUp(self)

    @skipUnlessDarwin
    @swiftTest
    def test_private_generic_type(self):
        """ Test that evaluating an expression without binding generic 
        parameters works for private generic types"""
        invisible_swift = self.getBuildArtifact("Private.swift")
        import shutil
        shutil.copyfile("Private.swift", invisible_swift)
        self.build()
        os.unlink(invisible_swift)
        os.unlink(self.getBuildArtifact("Private.swiftmodule"))
        os.unlink(self.getBuildArtifact("Private.swiftinterface"))

        target, process, _, _ = lldbutil.run_to_source_breakpoint(self, 
                'break here for struct', lldb.SBFileSpec('Public.swift'),
                extra_images=['Public'])
        # Make sure this fails without generic expression evaluation.
        self.expect("e --bind-generic-types true -- self", 
                    substrs=["Couldn't realize Swift AST type of self."], 
                    error=True)
        # Test that not binding works.
        self.expect("e --bind-generic-types false -- self", 
                    substrs=["Public.StructWrapper<T>", 
                             'name = "The invisible struct."'])
        # Test that the "auto" behavior also works.
        self.expect("e --bind-generic-types auto -- self", 
                    substrs=["Public.StructWrapper<T>", 
                             'name = "The invisible struct."'])
        # Test that the default (should be the auto option) also works.
        self.expect("e -- self", substrs=["Public.StructWrapper<T>", 
                                          'name = "The invisible struct."'])

        breakpoint = target.BreakpointCreateBySourceRegex(
            'break here for class', lldb.SBFileSpec('Public.swift'), None)
        lldbutil.continue_to_breakpoint(process, breakpoint)
        self.expect("e --bind-generic-types true -- self", 
                    substrs=["Couldn't realize Swift AST type of self."], 
                    error=True)
        self.expect("e --bind-generic-types false -- self", 
                    substrs=["Public.ClassWrapper<Private.InvisibleStruct>", 
                             'name = "The invisible struct."'])
        self.expect("e --bind-generic-types auto -- self", 
                    substrs=["Public.ClassWrapper<Private.InvisibleStruct>", 
                             'name = "The invisible struct."'])
        self.expect("e -- self", 
                    substrs=["Public.ClassWrapper<Private.InvisibleStruct>", 
                             'name = "The invisible struct."'])

        breakpoint = target.BreakpointCreateBySourceRegex(
            'break here for two generic parameters', lldb.SBFileSpec('Public.swift'), None)
        lldbutil.continue_to_breakpoint(process, breakpoint)
        self.expect("e --bind-generic-types true -- self", 
                    substrs=["Couldn't realize Swift AST type of self."], 
                    error=True)
        self.expect("e --bind-generic-types false -- self", 
                    substrs=["Public.TwoGenericParameters",
                             "<Private.InvisibleClass, Private.InvisibleStruct>", 
                             'name = "The invisible class."',
                             "someNumber = 42"])
        self.expect("e --bind-generic-types auto -- self", 
                    substrs=["Public.TwoGenericParameters",
                             "<Private.InvisibleClass, Private.InvisibleStruct>", 
                             'name = "The invisible class."',
                             "someNumber = 42"])
        self.expect("e -- self", 
                    substrs=["Public.TwoGenericParameters",
                             "<Private.InvisibleClass, Private.InvisibleStruct>", 
                             'name = "The invisible class."',
                             "someNumber = 42"])

        breakpoint = target.BreakpointCreateBySourceRegex(
            'break here for three generic parameters', lldb.SBFileSpec('Public.swift'), None)
        lldbutil.continue_to_breakpoint(process, breakpoint)
        self.expect("e --bind-generic-types true -- self", 
                    substrs=["Couldn't realize Swift AST type of self."], 
                    error=True)
        self.expect("e --bind-generic-types false -- self", 
                    substrs=["Public.ThreeGenericParameters",
                             "<T, U, V>", 
                             'name = "The invisible class."',
                             "someNumber = 42",
                             'name = "The invisible struct."',
                             "v = true"])
        self.expect("e --bind-generic-types auto -- self", 
                    substrs=["Public.ThreeGenericParameters",
                             "<T, U, V>", 
                             'name = "The invisible class."',
                             "someNumber = 42",
                             'name = "The invisible struct."',
                             "v = true"])
        self.expect("e -- self", 
                    substrs=["Public.ThreeGenericParameters",
                             "<T, U, V>", 
                             'name = "The invisible class."',
                             "someNumber = 42",
                             'name = "The invisible struct."',
                             "v = true"])

        breakpoint = target.BreakpointCreateBySourceRegex(
            'break here for four generic parameters', lldb.SBFileSpec('Public.swift'), None)
        lldbutil.continue_to_breakpoint(process, breakpoint)
        self.expect("e --bind-generic-types true -- self", 
                    substrs=["Couldn't realize Swift AST type of self."], 
                    error=True)
        self.expect("e --bind-generic-types false -- self", 
                    substrs=["Public.FourGenericParameters",
                             "<T, U, V, W>", 
                             'name = "The invisible struct."',
                             'name = "The invisible class."',
                             "someNumber = 42",
                             "v = 3 values", "One", "two", "three",
                             "w = 482"])
        self.expect("e --bind-generic-types auto -- self", 
                    substrs=["Public.FourGenericParameters",
                             "<T, U, V, W>", 
                             'name = "The invisible struct."',
                             'name = "The invisible class."',
                             "someNumber = 42",
                             "v = 3 values", "One", "two", "three",
                             "w = 482"])
        self.expect("e -- self", 
                    substrs=["Public.FourGenericParameters",
                             "<T, U, V, W>", 
                             'name = "The invisible struct."',
                             'name = "The invisible class."',
                             "someNumber = 42",
                             "v = 3 values", "One", "two", "three",
                             "w = 482"])

        breakpoint = target.BreakpointCreateBySourceRegex(
            'break here for non-generic', lldb.SBFileSpec('Public.swift'), None)
        lldbutil.continue_to_breakpoint(process, breakpoint)
        self.expect("e --bind-generic-types false -- self", 
                    substrs=["Could not evaluate the expression without binding generic types."], 
                    error=True)

        breakpoint = target.BreakpointCreateBySourceRegex(
            'break here for nested generic parameters', lldb.SBFileSpec('Public.swift'), None)
        lldbutil.continue_to_breakpoint(process, breakpoint)
        self.expect("e --bind-generic-types false -- self", 
                    substrs=["Could not evaluate the expression without binding generic types."], 
                    error=True)

        # Check that if both binding and not binding the generic type parameters fail, we report 
        # the "bind generic params" error message, as that's the default case that runs first.
        self.expect("e --bind-generic-types auto -- self", 
                    substrs=["Couldn't realize Swift AST type of self."], 
                    error=True)



"""
Test SBSection APIs.
"""


from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SectionAPITestCase(TestBase):
    @no_debug_info_test
    @skipIfXmlSupportMissing
    def test_get_target_byte_size(self):
        d = {"EXE": "b.out"}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)
        exe = self.getBuildArtifact("b.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # find the .data section of the main module
        mod = target.GetModuleAtIndex(0)
        data_section = None
        for s in mod.sections:
            sect_type = s.GetSectionType()
            if sect_type == lldb.eSectionTypeData:
                data_section = s
                break
            elif sect_type == lldb.eSectionTypeContainer:
                for i in range(s.GetNumSubSections()):
                    ss = s.GetSubSectionAtIndex(i)
                    sect_type = ss.GetSectionType()
                    if sect_type == lldb.eSectionTypeData:
                        data_section = ss
                        break

        self.assertIsNotNone(data_section)
        self.assertEqual(data_section.target_byte_size, 1)

    @no_debug_info_test
    @skipIfXmlSupportMissing
    def test_get_alignment(self):
        exe = self.getBuildArtifact("aligned.out")
        self.yaml2obj("aligned.yaml", exe)
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # exe contains a single section aligned to 0x1000
        section = target.modules[0].sections[0]
        self.assertEqual(section.GetAlignment(), 0x1000)
        self.assertEqual(section.alignment, 0x1000)

    @no_debug_info_test
    @skipIfXmlSupportMissing
    def test_compressed_section_data(self):
        exe = self.getBuildArtifact("compressed-sections.out")
        self.yaml2obj("compressed-sections.yaml", exe)
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # exe contains a single section with SHF_COMPRESSED. Check that
        # GetSectionData returns the uncompressed data and not the raw contents
        # of the section.
        section = target.modules[0].sections[0]
        section_data = section.GetSectionData().uint8s
        self.assertEqual(section_data, [0x20, 0x30, 0x40, 0x50, 0x60, 0x70, 0x80, 0x90])



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftFrameworkPaths(lldbtest.TestBase):

    @swiftTest
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(oslist=no_match(["macosx"]))
    def test_system_framework(self):
        """Test the discovery of framework search paths from framework dependencies."""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        log = self.getBuildArtifact("types.log")
        self.expect('log enable lldb types -f "%s"' % log)
        self.expect("expression -- 0")
        self.filecheck('platform shell cat "%s"' % log, __file__,
                       '--check-prefix=CHECK_SYS')
        # CHECK_SYS: SwiftASTContextForExpressions::LogConfiguration(){{.*}}/secret_path

    @swiftTest
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(oslist=no_match(["macosx"]))
    def test_module_context_framework_path(self):
        """Test the discovery of framework search paths from framework dependencies."""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        self.expect('settings set symbols.swift-validate-typesystem false')
        self.expect('settings set symbols.use-swift-typeref-typesystem false')
        log = self.getBuildArtifact("types.log")
        self.expect('log enable lldb types -f "%s"' % log)
        self.expect("expression -- d", substrs=["member"])
        self.filecheck('platform shell cat "%s"' % log, __file__,
                       '--check-prefix=CHECK_MOD')
        # CHECK_MOD: SwiftASTContextForModule("a.out")::LogConfiguration(){{.*}}other_secret_path



# TestSwiftPrivateTypeAlias.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that we correctly find private decls
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftPrivateTypeAlias(TestBase):
    @swiftTest
    def test_swift_private_typealias(self):
        """Test that we can correctly print variables whose types are private type aliases"""
        self.build()
        (target, process, thread, breakpoint1) = \
            lldbutil.run_to_source_breakpoint(
                self, 'breakpoint 1', lldb.SBFileSpec('main.swift'))
        breakpoint2 = target.BreakpointCreateBySourceRegex(
            'breakpoint 2', lldb.SBFileSpec('main.swift'))
        self.assertTrue(breakpoint1.GetNumLocations() > 0, VALID_BREAKPOINT)
        self.assertTrue(breakpoint2.GetNumLocations() > 0, VALID_BREAKPOINT)

        var = self.frame().FindVariable("i")
        lldbutil.check_variable(
            self,
            var,
            False,
            typename="a.MyStruct.IntegerType",
            value="123")

        process.Continue()
        threads = lldbutil.get_threads_stopped_at_breakpoint(process,
                                                             breakpoint2)
        self.assertTrue(len(threads) == 1)

        var = self.frame().FindVariable("a")
        dict_child_0 = var.GetChildAtIndex(0)
        child_0 = dict_child_0.GetChildAtIndex(0)
        child_1 = dict_child_0.GetChildAtIndex(1)
        lldbutil.check_variable(
            self, var, False, typename=
            "Swift.Dictionary<Swift.String, a.MyStruct.IntegerType>")
        lldbutil.check_variable(self, child_0, False, '"hello"')
        lldbutil.check_variable(self, child_1, False, value='234')




#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

try:
    from typing import Sequence, Union
    from ..ir import *
    from ._ods_common import get_default_loc_context

    from typing import Any, List, Union
except ImportError as e:
    raise RuntimeError("Error loading imports from extension module") from e


class AllocTensorOp:
    """Extends the bufferization.alloc_tensor op."""

    def __init__(
        self,
        tensor_type: Type,
        dynamic_sizes: Sequence[Value],
        copy: Value,
        size_hint: Value,
        escape: BoolAttr,
        *,
        loc=None,
        ip=None
    ):
        """Constructs an `alloc_tensor` with static and/or dynamic sizes."""
        context = get_default_loc_context(loc)
        attributes = {}
        if escape:
            attributes["escape"] = escape
        op = self.build_generic(
            results=[tensor_type],
            operands=[dynamic_sizes, copy, size_hint],
            attributes=attributes,
            loc=loc,
            ip=ip,
        )
        OpView.__init__(self, op)



"""
Test specific to MIPS
"""

import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AvoidBreakpointInDelaySlotAPITestCase(TestBase):
    @skipIf(archs=no_match(re.compile("mips*")))
    def test(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.expect("file " + exe, patterns=["Current executable set to .*a.out.*"])

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        breakpoint = target.BreakpointCreateByName("main", "a.out")
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() == 1, VALID_BREAKPOINT
        )

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        list = target.FindFunctions("foo", lldb.eFunctionNameTypeAuto)
        self.assertEqual(list.GetSize(), 1)
        sc = list.GetContextAtIndex(0)
        self.assertEqual(sc.GetSymbol().GetName(), "foo")
        function = sc.GetFunction()
        self.assertTrue(function)
        self.function(function, target)

    def function(self, function, target):
        """Iterate over instructions in function and place a breakpoint on delay slot instruction"""
        # Get the list of all instructions in the function
        insts = function.GetInstructions(target)
        print(insts)
        i = 0
        for inst in insts:
            if inst.HasDelaySlot():
                # Remember the address of branch instruction.
                branchinstaddress = inst.GetAddress().GetLoadAddress(target)

                # Get next instruction i.e delay slot instruction.
                delayinst = insts.GetInstructionAtIndex(i + 1)
                delayinstaddr = delayinst.GetAddress().GetLoadAddress(target)

                # Set breakpoint on delay slot instruction
                breakpoint = target.BreakpointCreateByAddress(delayinstaddr)

                # Verify the breakpoint.
                self.assertTrue(
                    breakpoint and breakpoint.GetNumLocations() == 1, VALID_BREAKPOINT
                )
                # Get the location from breakpoint
                location = breakpoint.GetLocationAtIndex(0)

                # Get the address where breakpoint is actually set.
                bpaddr = location.GetLoadAddress()

                # Breakpoint address should be adjusted to the address of
                # branch instruction.
                self.assertEqual(branchinstaddress, bpaddr)
                i += 1
            else:
                i += 1



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil

class TestCase(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    @skipIf(oslist=['windows', 'linux'])
    def test(self):
        """Test SBFrame.GetLanguageSpecificData() in async functions"""
        self.build()

        # This test uses "main" as a prefix for all functions, so that all will
        # be stopped at. Setting a breakpoint on "main" results in a breakpoint
        # at the start of each coroutine "funclet" function.
        target, process, thread, _ = lldbutil.run_to_name_breakpoint(self, "main")

        # Skip the real `main`. All other stops will be Swift async functions.
        self.assertEqual(thread.frames[0].name, "main")
        process.Continue()

        regex_bpno = lldbutil.run_break_set_by_regexp(self, "^main[[:digit:]]")

        while process.state == lldb.eStateStopped:
            thread = process.GetSelectedThread()
            frame = thread.frames[0]
            data = frame.GetLanguageSpecificData()
            is_async = data.GetValueForKey("IsSwiftAsyncFunction").GetBooleanValue()
            self.assertTrue(is_async)

            process.Continue()

        bkpt = target.FindBreakpointByID(regex_bpno)
        self.assertEqual(bkpt.GetHitCount(), bkpt.num_locations)
        for location in bkpt.locations:
            self.assertEqual(location.GetHitCount(), 1)



"""
Test that a nested template parameter works with simple template names.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class NestedTemplateTestCase(TestBase):
    def do_test(self, debug_flags):
        self.build(dictionary=debug_flags)
        self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        self.expect(
            "image lookup -A -t 'Inner<int>'",
            DATA_TYPES_DISPLAYED_CORRECTLY,
            substrs=["1 match found"],
        )

    @skipIf(compiler=no_match("clang"))
    @skipIf(compiler_version=["<", "15.0"])
    def test_simple_template_names(self):
        self.do_test(dict(TEST_CFLAGS_EXTRAS="-gsimple-template-names"))

    @skipIf(compiler=no_match("clang"))
    @skipIf(compiler_version=["<", "15.0"])
    def test_no_simple_template_names(self):
        self.do_test(dict(TEST_CFLAGS_EXTRAS="-gno-simple-template-names"))



"""Test lldb's expression evaluations and collect statistics."""

import sys
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbbench import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import configuration
from lldbsuite.test import lldbutil


class ExpressionEvaluationCase(BenchBase):
    def setUp(self):
        BenchBase.setUp(self)
        self.source = "main.cpp"
        self.line_to_break = line_number(self.source, "// Set breakpoint here.")
        self.count = 25

    @benchmarks_test
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr22274: need a pexpect replacement for windows",
    )
    def test_expr_cmd(self):
        """Test lldb's expression commands and collect statistics."""
        self.build()
        self.exe_name = "a.out"

        print()
        self.run_lldb_repeated_exprs(self.exe_name, self.count)
        print("lldb expr cmd benchmark:", self.stopwatch)

    def run_lldb_repeated_exprs(self, exe_name, count):
        import pexpect

        exe = self.getBuildArtifact(exe_name)

        # Set self.child_prompt, which is "(lldb) ".
        self.child_prompt = "(lldb) "
        prompt = self.child_prompt

        # Reset the stopwatch now.
        self.stopwatch.reset()
        for i in range(count):
            # So that the child gets torn down after the test.
            self.child = pexpect.spawn(
                "%s %s %s" % (lldbtest_config.lldbExec, self.lldbOption, exe)
            )
            child = self.child

            # Turn on logging for what the child sends back.
            if self.TraceOn():
                child.logfile_read = sys.stdout

            child.expect_exact(prompt)
            child.sendline(
                "breakpoint set -f %s -l %d" % (self.source, self.line_to_break)
            )
            child.expect_exact(prompt)
            child.sendline("run")
            child.expect_exact(prompt)
            expr_cmd1 = "expr ptr[j]->point.x"
            expr_cmd2 = "expr ptr[j]->point.y"

            with self.stopwatch:
                child.sendline(expr_cmd1)
                child.expect_exact(prompt)
                child.sendline(expr_cmd2)
                child.expect_exact(prompt)

            child.sendline("quit")
            try:
                self.child.expect(pexpect.EOF)
            except:
                pass

        self.child = None



"""
Set breakpoints on objective-c class and instance methods in foundation.
Also lookup objective-c data types and evaluate expressions.
"""

import os
import os.path
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

file_index = 0


class FoundationTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.main_source = "main.m"
        self.line = line_number(self.main_source, "// Set break point at this line.")

    def test_break(self):
        """Test setting objc breakpoints using '_regexp-break' and 'breakpoint set'."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Stop at +[NSString stringWithFormat:].
        break_results = lldbutil.run_break_set_command(
            self, "_regexp-break +[NSString stringWithFormat:]"
        )
        lldbutil.check_breakpoint_result(
            self,
            break_results,
            symbol_name="+[NSString stringWithFormat:]",
            num_locations=1,
        )

        # Stop at -[MyString initWithNSString:].
        lldbutil.run_break_set_by_symbol(
            self,
            "-[MyString initWithNSString:]",
            num_expected_locations=1,
            sym_exact=True,
        )

        # Stop at the "description" selector.
        lldbutil.run_break_set_by_selector(
            self, "description", num_expected_locations=1, module_name="a.out"
        )

        # Stop at -[NSAutoreleasePool release].
        break_results = lldbutil.run_break_set_command(
            self, "_regexp-break -[NSAutoreleasePool release]"
        )
        lldbutil.check_breakpoint_result(
            self,
            break_results,
            symbol_name="-[NSAutoreleasePool release]",
            num_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # First stop is +[NSString stringWithFormat:].
        self.expect(
            "thread backtrace",
            "Stop at +[NSString stringWithFormat:]",
            substrs=["Foundation`+[NSString stringWithFormat:]"],
        )

        self.runCmd("process continue")

        # Second stop is still +[NSString stringWithFormat:].
        self.expect(
            "thread backtrace",
            "Stop at +[NSString stringWithFormat:]",
            substrs=["Foundation`+[NSString stringWithFormat:]"],
        )

        self.runCmd("process continue")

        # Followed by a.out`-[MyString initWithNSString:].
        self.expect(
            "thread backtrace",
            "Stop at a.out`-[MyString initWithNSString:]",
            substrs=["a.out`-[MyString initWithNSString:]"],
        )

        self.runCmd("process continue")

        # Followed by -[MyString description].
        self.expect(
            "thread backtrace",
            "Stop at -[MyString description]",
            substrs=["a.out`-[MyString description]"],
        )

        self.runCmd("process continue")

        # Followed by the same -[MyString description].
        self.expect(
            "thread backtrace",
            "Stop at -[MyString description]",
            substrs=["a.out`-[MyString description]"],
        )

        self.runCmd("process continue")

        # Followed by -[NSAutoreleasePool release].
        self.expect(
            "thread backtrace",
            "Stop at -[NSAutoreleasePool release]",
            substrs=["Foundation`-[NSAutoreleasePool release]"],
        )

    # rdar://problem/8542091
    # rdar://problem/8492646
    def test_data_type_and_expr(self):
        """Lookup objective-c data types and evaluate expressions."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Stop at -[MyString description].
        lldbutil.run_break_set_by_symbol(
            self, "-[MyString description]", num_expected_locations=1, sym_exact=True
        )
        #        self.expect("breakpoint set -n '-[MyString description]", BREAKPOINT_CREATED,
        # startstr = "Breakpoint created: 1: name = '-[MyString description]',
        # locations = 1")

        self.runCmd("run", RUN_SUCCEEDED)

        self.runCmd("settings set target.prefer-dynamic-value no-dynamic-values")

        # The backtrace should show we stop at -[MyString description].
        self.expect(
            "thread backtrace",
            "Stop at -[MyString description]",
            substrs=["a.out`-[MyString description]"],
        )

        # Lookup objc data type MyString and evaluate some expressions.

        self.expect(
            "image lookup -t NSString",
            DATA_TYPES_DISPLAYED_CORRECTLY,
            substrs=['name = "NSString"', 'compiler_type = "@interface NSString'],
        )

        self.expect(
            "image lookup -t MyString",
            DATA_TYPES_DISPLAYED_CORRECTLY,
            substrs=[
                'name = "MyString"',
                'compiler_type = "@interface MyString',
                "NSString * str;",
                "NSDate * date;",
            ],
        )

        self.expect(
            "frame variable --show-types --scope",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["ARG: (MyString *) self"],
            patterns=["ARG: \(.*\) _cmd", "(objc_selector *)|(SEL)"],
        )

        # rdar://problem/8651752
        # don't crash trying to ask clang how many children an empty record has
        self.runCmd("frame variable *_cmd")

        # rdar://problem/8492646
        # test/foundation fails after updating to tot r115023
        # self->str displays nothing as output
        self.expect(
            "frame variable --show-types self->str",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(NSString *) self->str",
        )

        # rdar://problem/8447030
        # 'frame variable self->date' displays the wrong data member
        self.expect(
            "frame variable --show-types self->date",
            VARIABLES_DISPLAYED_CORRECTLY,
            startstr="(NSDate *) self->date",
        )

        # This should display the str and date member fields as well.
        self.expect(
            "frame variable --show-types *self",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["(MyString) *self", "(NSString *) str", "(NSDate *) date"],
        )

        # isa should be accessible.
        self.expect(
            "expression self->isa", VARIABLES_DISPLAYED_CORRECTLY, substrs=["Class)"]
        )

        # This should fail expectedly.
        self.expect(
            "expression self->non_existent_member",
            COMMAND_FAILED_AS_EXPECTED,
            error=True,
            substrs=[
                "error:",
                "'MyString' does not have a member named 'non_existent_member'",
            ],
        )

        # Use expression parser.
        self.runCmd("expression self->str")
        self.runCmd("expression self->date")

        # (lldb) expression self->str
        # error: instance variable 'str' is protected
        # error: 1 errors parsing expression
        #
        # (lldb) expression self->date
        # error: instance variable 'date' is protected
        # error: 1 errors parsing expression
        #

        self.runCmd("breakpoint delete 1")
        lldbutil.run_break_set_by_file_and_line(
            self, "main.m", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("process continue")

        # rdar://problem/8542091
        # test/foundation: expr -o -- my not working?
        #
        # Test new feature with r115115:
        # Add "-o" option to "expression" which prints the object description
        # if available.
        self.expect(
            "expression --object-description -- my",
            "Object description displayed correctly",
            patterns=["Hello from.*a.out.*with timestamp: "],
        )

    @add_test_categories(["pyapi"])
    def test_print_ivars_correctly(self):
        self.build()
        # See: <rdar://problem/8717050> lldb needs to use the ObjC runtime symbols for ivar offsets
        # Only fails for the ObjC 2.0 runtime.
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        break1 = target.BreakpointCreateByLocation(self.main_source, self.line)
        self.assertTrue(break1, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        thread = process.GetThreadAtIndex(0)
        if thread.GetStopReason() != lldb.eStopReasonBreakpoint:
            from lldbsuite.test.lldbutil import stop_reason_to_str

            self.fail(
                STOPPED_DUE_TO_BREAKPOINT_WITH_STOP_REASON_AS
                % stop_reason_to_str(thread.GetStopReason())
            )

        # Make sure we stopped at the first breakpoint.

        cur_frame = thread.GetFrameAtIndex(0)

        line_number = cur_frame.GetLineEntry().GetLine()
        self.assertEqual(line_number, self.line, "Hit the first breakpoint.")

        my_var = cur_frame.FindVariable("my")
        self.assertTrue(my_var, "Made a variable object for my")

        str_var = cur_frame.FindVariable("str")
        self.assertTrue(str_var, "Made a variable object for str")

        # Now make sure that the my->str == str:

        my_str_var = my_var.GetChildMemberWithName("str")
        self.assertTrue(my_str_var, "Found a str ivar in my")

        str_value = int(str_var.GetValue(), 0)

        my_str_value = int(my_str_var.GetValue(), 0)

        self.assertEqual(str_value, my_str_value, "Got the correct value for my->str")

    def test_expression_lookups_objc(self):
        """Test running an expression detect spurious debug info lookups (DWARF)."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Stop at -[MyString initWithNSString:].
        lldbutil.run_break_set_by_symbol(
            self,
            "-[MyString initWithNSString:]",
            num_expected_locations=1,
            sym_exact=True,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        global file_index
        # Log any DWARF lookups
        ++file_index
        logfile = os.path.join(
            self.getBuildDir(),
            "dwarf-lookups-" + self.getArchitecture() + "-" + str(file_index) + ".txt",
        )
        self.runCmd("log enable -f %s dwarf lookups" % (logfile))
        self.runCmd("expr self")
        self.runCmd("log disable dwarf lookups")

        def cleanup():
            if os.path.exists(logfile):
                os.unlink(logfile)

        self.addTearDownHook(cleanup)

        if os.path.exists(logfile):
            f = open(logfile)
            lines = f.readlines()
            num_errors = 0
            for line in lines:
                if "$__lldb" in line:
                    if num_errors == 0:
                        print(
                            "error: found spurious name lookups when evaluating an expression:"
                        )
                    num_errors += 1
                    print(line, end="")
            self.assertEqual(num_errors, 0, "Spurious lookups detected")
            f.close()



import lldb
import unittest
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestMultiprocess(GDBRemoteTestBase):
    def base_test(self, variant, follow_child=False):
        class MyResponder(MockGDBServerResponder):
            def __init__(self):
                super().__init__()
                self.first = True
                self.detached = None
                self.property = "{}-events+".format(variant)

            def qSupported(self, client_supported):
                assert "multiprocess+" in client_supported
                assert self.property in client_supported
                return "{};multiprocess+;{}".format(
                    super().qSupported(client_supported), self.property
                )

            def qfThreadInfo(self):
                return "mp400.10200"

            def cont(self):
                if self.first:
                    self.first = False
                    return "T0fthread:p400.10200;reason:{0};{0}:p401.10400;".format(
                        variant
                    )
                return "W00"

            def D(self, packet):
                self.detached = packet
                return "OK"

        self.server.responder = MyResponder()
        target = self.dbg.CreateTarget("")
        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))
        if follow_child:
            self.runCmd("settings set target.process.follow-fork-mode child")
        process = self.connect(target)
        self.assertEqual(process.GetProcessID(), 1024)
        process.Continue()
        self.assertRegex(
            self.server.responder.detached, r"D;0*400" if follow_child else r"D;0*401"
        )
        self.assertEqual(process.GetProcessID(), 1025 if follow_child else 1024)

    def test_fork(self):
        self.base_test("fork")

    def test_vfork(self):
        self.base_test("vfork")

    def test_fork_follow_child(self):
        self.base_test("fork", follow_child=True)

    def test_vfork_follow_child(self):
        self.base_test("vfork", follow_child=True)



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftGenericFunction(lldbtest.TestBase):
    @swiftTest
    def test(self):
        """Test display of generic function names"""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_name_breakpoint(
            self, 'f')

        # Live process:
        stream = lldb.SBStream()
        self.frame().GetDescription(stream)
        desc = stream.GetData()
        # It's debatable whether C's generic parameter should be displayed here.
        self.assertIn("C.f<Int>(t=1, u=2)", desc)

        # Dead process + debug info:
        process.Kill()
        stream = lldb.SBStream()
        bkpt.GetLocationAtIndex(0).GetDescription(stream, 1)
        desc = stream.GetData()
        self.assertIn("C.f<T>(_:_:)", desc)

        # Demangling only:
        fs = target.FindFunctions("f")
        self.assertTrue(fs)
        desc = fs[0].GetFunction().GetDisplayName()
        self.assertIn("C.f<τ_0_0>(_:_:)", desc)




"""
Test lldb ability to unwind a stack with a function containing a call to the
'__builtin_trap' intrinsic, which GCC (4.6) encodes to an illegal opcode.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class BuiltinTrapTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    # gcc generates incorrect linetable
    @expectedFailureAll(archs="arm", compiler="gcc", triple=".*-android")
    @expectedFailureAll(archs=["aarch64"], oslist=no_match(["freebsd", "linux"]))
    @skipIfWindows
    def test_with_run_command(self):
        """Test that LLDB handles a function with __builtin_trap correctly."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # print backtrace, expect both 'bar' and 'main' functions to be listed
        self.expect("bt", substrs=["bar", "main"])

        # go up one frame
        self.runCmd("up", RUN_SUCCEEDED)

        # evaluate a local
        self.expect("expression foo", substrs=["= 5"])



"""
Tests the exit code/description coming from the debugserver.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    @skipUnlessDarwin
    @skipIfOutOfTreeDebugserver
    def test_abort(self):
        self.build()
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        process = target.LaunchSimple(None, None, None)
        # Continue until process is terminated.
        process.Continue()
        # Test for the abort signal code.
        self.assertEqual(process.GetExitStatus(), 6)
        # Test for the exit code description.
        self.assertEqual(process.GetExitDescription(), "Terminated due to signal 6")



"""
Tests const static data members as specified by C++11 [class.static.data]p3
with (u)int128_t types.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    # int128 is not available on 32-bit ARM.
    @skipIf(archs=["arm"])
    def test_int128(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        # Try to use the (u)int128_t data members which are not supported at
        # the moment. Just verify that LLDB doesn't report an incorrect value
        # for them and just treats them as normal variables (which will lead
        # to linker errors as they are not defined anywhere).
        self.expect(
            "expr A::int128_max", error=True, substrs=["Couldn't look up symbols:"]
        )
        self.expect(
            "expr A::uint128_max", error=True, substrs=["Couldn't look up symbols:"]
        )
        self.expect(
            "expr A::int128_min", error=True, substrs=["Couldn't look up symbols:"]
        )
        self.expect(
            "expr A::uint128_min", error=True, substrs=["Couldn't look up symbols:"]
        )



"""
Test that dynamic values update their child count correctly
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class DynamicValueChildCountTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        # Find the line number to break for main.c.

        self.main_third_call_line = line_number(
            "pass-to-base.cpp", "// Break here and check b has 0 children"
        )
        self.main_fourth_call_line = line_number(
            "pass-to-base.cpp", "// Break here and check b still has 0 children"
        )
        self.main_fifth_call_line = line_number(
            "pass-to-base.cpp", "// Break here and check b has one child now"
        )
        self.main_sixth_call_line = line_number(
            "pass-to-base.cpp", "// Break here and check b has 0 children again"
        )

    @add_test_categories(["pyapi"])
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24663")
    def test_get_dynamic_vals(self):
        """Test fetching C++ dynamic values from pointers & references."""
        """Get argument vals for the call stack when stopped on a breakpoint."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target from the debugger.

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set up our breakpoints:

        third_call_bpt = target.BreakpointCreateByLocation(
            "pass-to-base.cpp", self.main_third_call_line
        )
        self.assertTrue(third_call_bpt, VALID_BREAKPOINT)
        fourth_call_bpt = target.BreakpointCreateByLocation(
            "pass-to-base.cpp", self.main_fourth_call_line
        )
        self.assertTrue(fourth_call_bpt, VALID_BREAKPOINT)
        fifth_call_bpt = target.BreakpointCreateByLocation(
            "pass-to-base.cpp", self.main_fifth_call_line
        )
        self.assertTrue(fifth_call_bpt, VALID_BREAKPOINT)
        sixth_call_bpt = target.BreakpointCreateByLocation(
            "pass-to-base.cpp", self.main_sixth_call_line
        )
        self.assertTrue(sixth_call_bpt, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        b = self.frame().FindVariable("b").GetDynamicValue(lldb.eDynamicCanRunTarget)
        self.assertEqual(b.GetNumChildren(), 0, "b has 0 children")
        self.runCmd("continue")
        self.assertEqual(b.GetNumChildren(), 0, "b still has 0 children")
        self.runCmd("continue")
        self.assertNotEqual(b.GetNumChildren(), 0, "b now has 1 child")
        self.runCmd("continue")
        self.assertEqual(b.GetNumChildren(), 0, "b didn't go back to 0 children")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftNoRuntime(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @swiftTest
    def test(self):
        """Test running a Swift expression in a C program"""
        self.build()
        self.expect("b test")
        _, process, _, _ = lldbutil.run_to_name_breakpoint(self, "main")
        options = lldb.SBExpressionOptions()
        options.SetIgnoreBreakpoints(False);
        value = self.frame().EvaluateExpression("test()", options)
        self.assertIn("breakpoint", str(value.GetError()))
        process.Kill()

